'From Squeak2.9alpha of 13 June 2000 [latest update: #2440] on 13 August 2000 at 1:04:25 pm'!
InterpreterPlugin subclass: #TimePlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-DateTime'!

!TimePlugin commentStamp: 'ls 6/30/2000 10:48' prior: 0!
A plugin providing various time facilities.  It should compile as-is on most platforms.!

!DateAndTime methodsFor: 'converting' stamp: 'ls 6/30/2000 11:30'!
asLocal
	"Answer a <DateAndTime> that represents the same UTC time as the 
	receiver, but in the local time specified by the implementation."
	| referenceTime |
	self flag: #xxx. "this implementation assumes that the timezone currently in effect should be used on the receiver.  This causes funny effects, but should be 'correct' (whatever 'correct' means for local times)"
	referenceTime := DateAndTime now.
	^ self class
		newRationalLocalDate: localDate
		time: localTime + referenceTime offset
		offset: referenceTime offset
		timeZoneName: referenceTime timeZoneName
		timeZoneAbbrev: referenceTime timeZoneAbbreviation! !

!DateAndTime methodsFor: 'converting' stamp: 'ls 6/30/2000 11:25'!
asUTC
	"Answer a <DateAndTime> that represents the same absolute time as the 
	receiver but in the local time UTC."
	^ self class
		newRationalLocalDate: localDate
		time: localTime - self offset
		offset: Duration zero
		timeZoneName: 'Universal Coordinated Time'
		timeZoneAbbrev: 'UTC'! !


!DateAndTime class methodsFor: 'constants' stamp: 'ls 6/30/2000 11:42'!
now
	"Answer a <DateAndTime> representing the current date and time."
	| timeArray localDate utcDate localTime utcTime seconds date time |
	timeArray := self primCurrentTime.
	timeArray ifNil: [
		"use the old primitive, and assume UTC time"
		seconds := self primSecondsClock.
		date := Date fromSeconds: seconds.
		time := Time fromSeconds: seconds \\ 86400. 
		timeArray := {
			date year.
			date dayOfYear.
			time hours.
			time minutes.
			time seconds.
			date year.
			date dayOfYear.
			time hours.
			time minutes.
			time seconds.
			'Universal Coordinated Time'.
			'UTC' }.
	].

	localDate := Date
		newDay: (timeArray at: 2)
		year: (timeArray at: 1).
	utcDate := Date
		newDay: (timeArray at: 7)
		year: (timeArray at: 6).

	localTime := Time new
		hours: (timeArray at: 3)
		minutes: (timeArray at: 4)
		seconds: (timeArray at: 5).

	utcTime := Time new
		hours: (timeArray at: 8)
		minutes: (timeArray at: 9)
		seconds: (timeArray at: 10).

	^ self
		newRationalLocalDate: (localDate)
		time: (Duration seconds: localTime asSeconds)
		offset: (Duration seconds: (localDate asSeconds + localTime asSeconds - utcDate asSeconds - utcTime asSeconds))
		timeZoneName: (timeArray at: 11)
		timeZoneAbbrev: (timeArray at: 12) ! !

!DateAndTime class methodsFor: 'primitives' stamp: 'ls 6/30/2000 12:56'!
primCurrentTime
	^self primCurrentTimeInto: (Array new: 12).
! !

!DateAndTime class methodsFor: 'primitives' stamp: 'ls 8/10/2000 16:08'!
primCurrentTimeInto: anArray
	^StandardSystemClock primCurrentTimeInto: anArray! !

!DateAndTime class methodsFor: 'primitives' stamp: 'ls 8/10/2000 16:09'!
primSecondsClock
	^StandardSystemClock primSecondsClock! !


!TimePlugin methodsFor: 'time' stamp: 'ls 6/30/2000 11:07'!
primitiveCurrentTime
	| responseArray theTime currentTZName tzNameForSmalltalk |
	self export: true.
	self var: 'theTime' type: 'time_t'.
	self var: 'currentTZName' type: 'unsigned char *'.

	theTime := 0.  theTime.          "get rid of compiler warnings"
	self cCode: 'time(&theTime)'.

	"copy over the time itself"
	self stuffTime: (self cCode: 'gmtime(&theTime)') at: 6.
	self stuffTime: (self cCode: 'localtime(&theTime)') at: 1.

	"copy in the timezone information"
	currentTZName := self cCode: 'tzname[daylight]'.

	tzNameForSmalltalk := self stringFromC: currentTZName.
	interpreterProxy
		stObject: (interpreterProxy stackObjectValue: 0)
		at: 11 put: tzNameForSmalltalk.
	interpreterProxy
		stObject: (interpreterProxy stackObjectValue: 0)
		at: 12 put: tzNameForSmalltalk.

	interpreterProxy failed ifFalse: [
		responseArray := interpreterProxy stackObjectValue: 0.
		interpreterProxy pop: 2.
		interpreterProxy push: responseArray ].

! !

!TimePlugin methodsFor: 'time' stamp: 'ls 6/30/2000 10:56'!
stringFromC: cString
	"create a Smalltalk string with value the given string, and return an oop for it.  Return nil and signal failure if anything goes wrong"
	|stStringIndexedFields stString |
	self var: 'stStringIndexedFields' type: 'unsigned char *'.
	self var: 'cString' type: 'unsigned char *'.
	self export: false.

	stString := interpreterProxy
		instantiateClass: interpreterProxy classString
		indexableSize: (self cCode: 'strlen(cString)').
	stStringIndexedFields := interpreterProxy firstIndexableField: stString.  stStringIndexedFields.
	interpreterProxy failed ifTrue: [ ^nil ].
	self cCode: 'strcpy(stStringIndexedFields, cString)'.
	^stString! !

!TimePlugin methodsFor: 'time' stamp: 'ls 6/27/2000 16:37'!
stuffTime: aTime at: index
	"encode the given time into an integer and stuff it into the response array (which should be on the top of the stack).  this will fill in 5 fields of the array, starting with the index-th"
	| responseArray |
	self var: 'aTime' type: 'struct tm *'.

	responseArray := interpreterProxy stackValue: 0.

	interpreterProxy stObject: responseArray at: index put: (interpreterProxy integerObjectOf: (self cCode: 'aTime->tm_year+1900')).
	interpreterProxy stObject: responseArray at: index+1 put: (interpreterProxy integerObjectOf: (self cCode: 'aTime->tm_yday+1')).
	interpreterProxy stObject: responseArray at: index+2 put: (interpreterProxy integerObjectOf: (self cCode: 'aTime->tm_hour')).
	interpreterProxy stObject: responseArray at: index+3 put: (interpreterProxy integerObjectOf: (self cCode: 'aTime->tm_min')).
	interpreterProxy stObject: responseArray at: index+4 put: (interpreterProxy integerObjectOf: (self cCode: 'aTime->tm_sec')).

! !


!TimePlugin reorganize!
('time' primitiveCurrentTime stringFromC: stuffTime:at:)
!

