'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 11:58:53 am'!
"Change Set:		Islands-blocks1
Date:			21 June 2000
Author:			Lex Spoon

Adds all the basic mechanisms for switching [...] constructs so that they return a wrapped BlockCotnext, instead of a BlockContext itself.

One important detail to note is that Processes are now created with a new idiom.  Instead of [...] fork or [...] newProcess, it should be:

	Process forBlock: [...]
or
	Process fork: [...]
"!

ParseNode subclass: #BlockNode
	instanceVariableNames: 'arguments statements returns nArgsNode size remoteCopyNode temporaries restrictedProxyNode '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Compiler'!
Object subclass: #RestrictedBlock
	instanceVariableNames: 'block '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sandbox-Proxies'!

!RestrictedBlock commentStamp: '<historical>' prior: 0!
A full-featured block of code.  It allows invocation via value, value:, value:value:, etc, but it also has various control-flow features.

This is the object that is returned by a [] construct once the Sandbox code has been loaded; actual BlockContexts are not accessed directly by most code any longer.!
Object makeSelector: #hasLiteral: privilaged: false!
!Object methodsFor: 'testing' stamp: 'ls 6/20/2000 17:10'!
hasLiteral: literal
	"Answer true if literal is identical to any literal in this array, even 
	if imbedded in further array structure. This method is only intended 
	for private use by CompiledMethod hasLiteralSymbol:"
	^self == literal! !

Array makeSelector: #hasLiteral: privilaged: false!
!Array methodsFor: 'private' stamp: 'ls 6/20/2000 17:10'!
hasLiteral: literal
	"Answer true if literal is identical to any literal in this array, even 
	if imbedded in further array structure. This method is only intended 
	for private use by CompiledMethod hasLiteralSymbol:"

	| |
	1 to: self size do: 
		[:index | 
		((self at: index) hasLiteral: literal) ifTrue: [^ true] ].
	^ false! !

AsyncFile makeSelector: #readByteCount:fromFilePosition:onCompletionDo: privilaged: false!
!AsyncFile methodsFor: 'as yet unclassified' stamp: 'ls 6/21/2000 13:38'!
readByteCount: byteCount fromFilePosition: fPosition onCompletionDo: aBlock
	"Start a read operation to read byteCount's from the given position in this file. and fork a process to await its completion. When the operation completes, evaluate the given block. Note that, since the completion block may run asynchronous, the client may need to use a SharedQueue or a semaphore for synchronization."

	| buffer n |
	buffer _ String new: byteCount.
	self primReadStart: fileHandle fPosition: fPosition count: byteCount.
	"here's the process that awaits the results:"
	Process fork: [
		[	semaphore wait.
		  	n _ self primReadResult: fileHandle intoBuffer: buffer at: 1 count: byteCount.
		  	n = Busy.
		] whileTrue.  "loop while busy in case the semaphore had excess signals"
		n = Error ifTrue: [^ self error: 'asynchronous read operation failed'].
		aBlock value: buffer.
	] at: Processor userInterruptPriority.
! !
AsyncFile makeSelector: #writeBuffer:atFilePosition:onCompletionDo: privilaged: false!
!AsyncFile methodsFor: 'as yet unclassified' stamp: 'ls 6/21/2000 13:39'!
writeBuffer: buffer atFilePosition: fPosition onCompletionDo: aBlock
	"Start an operation to write the contents of the buffer at given position in this file, and fork a process to await its completion. When the write completes, evaluate the given block. Note that, since the completion block runs asynchronously, the client may need to use a SharedQueue or a semaphore for synchronization."

	| n |
	self primWriteStart: fileHandle
		fPosition: fPosition
		fromBuffer: buffer
		at: 1
		count: buffer size.
	"here's the process that awaits the results:"
	Process fork: [
		[	semaphore wait.
		  	n _ self primWriteResult: fileHandle.
		  	n = Busy.
		] whileTrue.  "loop while busy in case the semaphore had excess signals"
		n = Error ifTrue: [^ self error: 'asynchronous write operation failed'].
		n = buffer size ifFalse: [^ self error: 'did not write the entire buffer'].
		aBlock value.
	] at: Processor userInterruptPriority.
! !

BlockNode makeSelector: #emitForValue:on: privilaged: false!
!BlockNode methodsFor: 'code generation' stamp: 'ls 7/7/2000 16:47'!
emitForValue: stack on: aStream

	aStream nextPut: LdThisContext.
	stack push: 1.
	nArgsNode emitForValue: stack on: aStream.
	remoteCopyNode
		emit: stack
		args: 1
		on: aStream.
	"Force a two byte jump."
	self emitLong: size code: JmpLong on: aStream.
	stack push: arguments size.
	arguments reverseDo: [:arg | arg emitStorePop: stack on: aStream].
	self emitForEvaluatedValue: stack on: aStream.
	self returns ifFalse: [aStream nextPut: EndRemote].
	stack pop: 1.

	self generateRestrictedBlocks ifTrue: [
		restrictedProxyNode
			emit: stack
			args: 0
			on: aStream ].! !
BlockNode makeSelector: #generateRestrictedBlocks privilaged: false!
!BlockNode methodsFor: 'code generation' stamp: 'ls 7/10/2000 15:59'!
generateRestrictedBlocks
	^false! !
BlockNode makeSelector: #sizeForValue: privilaged: false!
!BlockNode methodsFor: 'code generation' stamp: 'ls 7/7/2000 16:47'!
sizeForValue: encoder
	| sizeToReturn |
	nArgsNode _ encoder encodeLiteral: arguments size.
	remoteCopyNode _ encoder encodeSelector: #blockCopy:.
	restrictedProxyNode _ encoder encodeSelector: #restrictedProxy.
	size _ (self sizeForEvaluatedValue: encoder)
				+ (self returns ifTrue: [0] ifFalse: [1]). "endBlock"
	arguments _ arguments collect:  "Chance to prepare debugger remote temps"
				[:arg | arg asStorableNode: encoder].
	arguments do: [:arg | size _ size + (arg sizeForStorePop: encoder)].
	sizeToReturn _ 1 + (nArgsNode sizeForValue: encoder) 
		+ (remoteCopyNode size: encoder args: 1 super: false) + 2 + size.

	self generateRestrictedBlocks ifTrue: [
		sizeToReturn _ sizeToReturn + (restrictedProxyNode size: encoder args: 0 super: false) ].

	^sizeToReturn! !

CharRecog makeSelector: #recogPar privilaged: false!
!CharRecog methodsFor: 'historical & disused' stamp: 'ls 6/21/2000 13:35'!
recogPar | prv cdir result features char r s t dir |

"Inits"				(p _ Pen new) defaultNib: 1; down.
	"for points"		pts _ ReadWriteStream on: #().

"Event Loop"	
		[Sensor anyButtonPressed] whileFalse: [(Sensor mousePoint x < 50) ifTrue: [^''].].

"First-Time"			pts reset.		
"will hold features"		ftrs _ ''.

					  (Sensor anyButtonPressed) ifTrue:
						[pts nextPut: (bmin _ bmax _ t _ s _ sts _ Sensor mousePoint).
						p place: sts. cdir _ nil.

"Each-Time"		[Sensor anyButtonPressed] whileTrue:
						[
"ink raw input"			p goto: (r _ Sensor mousePoint).
"smooth it"				s _ (0.5*s) + (0.5*r).
"thin the stream"		((s x - t x) abs > 3 or:[(s y - t y) abs > 3]) ifTrue:
							[ pts nextPut: t. 
"bounding box"			bmin _ bmin min: s. bmax _ bmax max: s.
"get current dir"				dir _ (self fourDirsFrom: t to: s). t _ s.
							dir ~= ' dot... ' ifTrue: [
"store new dirs"					cdir ~= dir ifTrue: [ftrs _ ftrs, (cdir _ dir)]].
"for inked t's" 			p place: t; go: 1; place: r.
							].
 "End Each-Time Loop"	].

"Last-Time"	
"start a new recog for next point"	Process fork: [CharRecog new recognize] .

"save last points"		pts nextPut: t; nextPut: r.
"find rest of features"	features _ self extractFeatures.
"find char..."			char _ CharacterDictionary at: features ifAbsent:
"...or get from user"			[ result _ FillInTheBlank request:
							 'Not recognized. type char, or type ~: ', features.
"ignore or..."				result = '~' ifTrue: ['']
"...enter new char"			ifFalse: [CharacterDictionary at: features put: result. result]].

"control the editor"		(char = 'cr' ifTrue: [Transcript cr] ifFalse:
						[char = 'bs' ifTrue: [Transcript bs] ifFalse:
						[char = 'tab' ifTrue:[Transcript tab] ifFalse:
						[Transcript show: char]]]). 

"End First-Time Loop"	]. 



			   
 ! !

CompiledMethod makeSelector: #frameSize privilaged: true!
!CompiledMethod methodsFor: 'accessing' stamp: 'ls 8/11/2000 09:53'!
frameSize
	"Answer the size of temporary frame needed to run the receiver."
	"NOTE:  Versions 2.7 and later use two sizes of contexts."

	(self header noMask: 16r20000)
		ifTrue: [^ !!SmallFrame]
		ifFalse: [^ !!LargeFrame]

! !
CompiledMethod makeSelector: #hasLiteralThorough: privilaged: false!
!CompiledMethod methodsFor: 'literals' stamp: 'ls 6/20/2000 17:13'!
hasLiteralThorough: literal
	"Answer true if any literal in this method is literal,
	even if embedded in array structure."

	| |
	2 to: self numLiterals + 1 do: 
		[:index | 
		((self objectAt: index) hasLiteral: literal) ifTrue: [^ true]. ].
	^ false! !

ComplexProgressIndicator makeSelector: #withProgressDo: privilaged: false!
!ComplexProgressIndicator methodsFor: 'as yet unclassified' stamp: 'ls 6/21/2000 13:39'!
withProgressDo: aBlock

	| safetyFactor totals |

	targetMorph ifNil: [
		targetMorph _ RectangleMorph new
			position: Sensor cursorPoint;
			extent: 120@120;
			openInWorld		"need to see about deleting later"
	].
	translucentMorph _ TranslucentProgessMorph new
		bounds: targetMorph bounds;
		openInWorld.
	stageCompleted _ 0.
	safetyFactor _ 1.1.	"better to guess high than low"
	translucentMorph setProperty: #progressStageNumber toValue: 1.
	totals _ self loadingHistoryDataForKey: 'total'.
	newRatio _ 1.0.
	estimate _ totals size < 2 ifTrue: [
		5000
	] ifFalse: [
		(totals sum - totals max) / (totals size - 1 max: 1) * safetyFactor.
	].
	formerProject _ Project current.
	formerWorld _ World.
	formerProcess _ Processor activeProcess.
	start _ Time millisecondClockValue.
	Process fork: [
		[stageCompleted < 999 and: [formerProject == Project current and: 
					[formerWorld == World]]] whileTrue: [
			translucentMorph setProperty: #revealTimes toValue: 
					{(Time millisecondClockValue - start max: 1). (estimate * newRatio max: 1)}.
			translucentMorph changed.
			"World displayWorldSafely."

			(Delay forMilliseconds: 100) wait.
		].
		translucentMorph removeProperty: #revealTimes.
		self loadingHistoryAt: 'total' add: (Time millisecondClockValue - start max: 1).
		formerWorld == World ifTrue: [
			translucentMorph delete.
			userSuppliedMorph ifNil: [targetMorph delete].
		] ifFalse: [
			translucentMorph privateDeleteWithAbsolutelyNoSideEffects.
			userSuppliedMorph ifNil: [targetMorph privateDeleteWithAbsolutelyNoSideEffects].
		].
	] at: 6.

	[
		aBlock 
			on: ProgressInitiationException
			do: [ :ex | 
				ex sendNotificationsTo: [ :min :max :curr |
					"ignore this as it is inaccurate"
				].
			].
	] on: ProgressNotification do: [ :note |
		stageCompleted _ (note messageText findTokens: ' ') first asNumber.
		cumulativeStageTime _ Time millisecondClockValue - start max: 1.
		prevData _ self loadingHistoryDataForKey: stageCompleted.
		prevData isEmpty ifFalse: [
			newRatio _ (cumulativeStageTime / (prevData average max: 1)) asFloat.
		].
		self loadingHistoryAt: stageCompleted add: cumulativeStageTime.
		translucentMorph setProperty: #progressStageNumber toValue: stageCompleted + 1.
		note resume.
	].

	stageCompleted _ 999.	"we'll rarely get here"

! !

ConnectionQueue makeSelector: #initPortNumber:queueLength: privilaged: false!
!ConnectionQueue methodsFor: 'private' stamp: 'ls 6/21/2000 13:41'!
initPortNumber: anInteger queueLength: queueLength
	"Private!! Initialize the receiver to listen on the given port number. Up to queueLength connections will be queued."

	portNumber _ anInteger.
	maxQueueLength _ queueLength.
	connections _ OrderedCollection new.
	accessSema _ Semaphore forMutualExclusion.
	socket _ nil.
	process _ Process forBlock: [self listenLoop].
	process priority: Processor highIOPriority.
	process resume.
! !

ContextPart makeSelector: #restrictedBlockCopy: privilaged: true!
!ContextPart methodsFor: 'controlling' stamp: 'ls 8/14/2000 12:01'!
restrictedBlockCopy: numArgs 
	"Primitive. Distinguish a block of code from its enclosing method by 
	creating a new BlockContext for that block. The compiler inserts into all 
	methods that contain blocks the bytecodes to send the message 
	blockCopy:. Do not use blockCopy: in code that you write!! Only the 
	compiler can decide to send the message blockCopy:. Fail if numArgs is 
	not a SmallInteger. Optional. No Lookup. See Object documentation 
	whatIsAPrimitive."
	| block |
	block := (!!BlockContext newForMethod: self home method)
		home: self home
		startpc: pc + 2
		nargs: numArgs.
	^!!RestrictedBlock on: block
! !

BlockContext makeSelector: #restrictedProxy privilaged: true!
!BlockContext methodsFor: 'islands' stamp: 'ls 8/14/2000 10:14'!
restrictedProxy
	^!!RestrictedBlock on: self! !

ControlManager makeSelector: #activeController: privilaged: true!
!ControlManager methodsFor: 'accessing' stamp: 'ls 6/21/2000 13:42'!
activeController: aController 
	"Set aController to be the currently active controller. Give the user 
	control in it."
	<primitive: 19> "Simulation guard"
	activeController _ aController.
	(activeController == screenController)
		ifFalse: [self promote: activeController].
	activeControllerProcess _ Process forBlock: 
			[activeController startUp.
			self searchForActiveController].
	activeControllerProcess priority: Processor userSchedulingPriority.
	activeControllerProcess resume! !

Delay class makeSelector: #startTimerInterruptWatcher privilaged: false!
!Delay class methodsFor: 'timer process' stamp: 'ls 6/21/2000 13:42'!
startTimerInterruptWatcher
	"Reset the class variables that keep track of active Delays and re-start the timer interrupt watcher process. Any currently scheduled delays are forgotten."
	"Delay startTimerInterruptWatcher"

	| p |
	self primSignal: nil atMilliseconds: 0.
	TimingSemaphore == nil
		ifFalse: [TimingSemaphore terminateProcess].
	TimingSemaphore _ Semaphore new.
	AccessProtect _ Semaphore forMutualExclusion.
	SuspendedDelays _ 
		SortedCollection sortBlock: 
			[:d1 :d2 | d1 resumptionTime <= d2 resumptionTime].
	ActiveDelay _ nil.
	p _ Process forBlock: [self timerInterruptWatcher].
	p priority: Processor timingPriority.
	p resume.
! !
Delay class makeSelector: #testDelayOf:for:rect: privilaged: false!
!Delay class methodsFor: 'example' stamp: 'ls 6/21/2000 13:39'!
testDelayOf: delay for: testCount rect: r
	"Delay testDelayOf: 100 for: 20 rect: (10@10 extent: 30@30).
	 Delay testDelayOf: 400 for: 20 rect: (50@10 extent: 30@30)."

	| onDelay offDelay |
	onDelay _ Delay forMilliseconds: 50.
	offDelay _ Delay forMilliseconds: delay - 50.
	Display fillBlack: r.
	Process fork: [1 to: testCount do: [:i |
		Display fillWhite: r.
		onDelay wait.
		Display reverse: r.
		offDelay wait].
	] at: Processor userInterruptPriority.
! !

Exception class makeSelector: #findHandlerFor:from: privilaged: true!
!Exception class methodsFor: 'finding handler' stamp: 'ls 8/14/2000 12:01'!
findHandlerFor: exception from: startCtx
	"search for a handler for the given exception, starting from the given context"
	| ctx handler |
	ctx := startCtx.
	[ctx == nil]
		whileFalse:
			[ctx isHandlerContext
				ifTrue:
					[handler := ctx tempAt: 1. "the first argument"
						((handler handles: exception) and: [|block realBlock| block := ctx tempAt: 2. realBlock :=  (!!ObjectInspector realBlockFor: block). realBlock sender == nil])
							ifTrue: [^ctx]].
			ctx := ctx sender].
       ^nil

! !

FaceMorph makeSelector: #step privilaged: false!
!FaceMorph methodsFor: 'stepping and presenter' stamp: 'ls 6/21/2000 13:37'!
step
	| amount |
	super step.
	10 atRandom = 1
		ifTrue: [Process fork:  [self lips perform: #(smile horror surprise sad grin) atRandom.
				 (Delay forMilliseconds: 2000 atRandom) wait.
				 self lips perform: #(neutral neutral smile sad) atRandom]].
	5 atRandom = 1
		ifTrue: [Process fork: [self closeEyelids.
				 (Delay forMilliseconds: 180) wait.
				 self openEyelids.
				 2 atRandom = 1 ifTrue: [self lookAtFront]].
				^ self].
	"20 atRandom = 1 ifTrue: [(self perform: #(leftEye rightEye) atRandom) closeEyelid]."
	20 atRandom = 1 ifTrue: [amount _ (0.2 to: 1.0 by: 0.01) atRandom.
							 self leftEye openness: amount. self rightEye openness: amount].
	3 atRandom = 1 ifTrue: [self lookAtHand. ^ self].
	3 atRandom = 1 ifTrue: [self lookAtFront. ^ self].
	3 atRandom = 1 ifTrue: [self lookAtMorph: self world submorphs atRandom]! !

FlashMorphReader makeSelector: #processFileAsync privilaged: false!
!FlashMorphReader methodsFor: 'reading' stamp: 'ls 6/21/2000 13:36'!
processFileAsync
	"Read and process the entire file"
	self processHeader ifFalse:[^nil].
	player sourceUrl:'dummy'.
	Process fork: [self processFileContents].
	^player! !

InputSensor makeSelector: #installInterruptWatcher privilaged: false!
!InputSensor methodsFor: 'user interrupts' stamp: 'ls 6/21/2000 13:42'!
installInterruptWatcher
	"Initialize the interrupt watcher process. Terminate the old process if any."
	"Sensor installInterruptWatcher"

	InterruptWatcherProcess == nil ifFalse: [InterruptWatcherProcess terminate].
	InterruptSemaphore _ Semaphore new.
	InterruptWatcherProcess _ Process forBlock: [self userInterruptWatcher].
	InterruptWatcherProcess priority: Processor lowIOPriority.
	InterruptWatcherProcess resume.
	self primInterruptSemaphore: InterruptSemaphore.! !

MIDISynth makeSelector: #startMIDITracking privilaged: false!
!MIDISynth methodsFor: 'as yet unclassified' stamp: 'ls 6/21/2000 13:42'!
startMIDITracking

	midiParser ifNil: [^ self].
	midiParser midiPort ifNil: [^ self].
	midiParser midiPort ensureOpen.
	self stopMIDITracking.
	SoundPlayer useShortBuffer.
	process _ Process forBlock: [self midiTrackingLoop].
	process priority: Processor userInterruptPriority.
	process resume.
! !

MailNotifier makeSelector: #messageCount privilaged: false!
!MailNotifier methodsFor: 'accessing' stamp: 'ls 6/21/2000 13:39'!
messageCount
	"if no socket, try to connect in background, when done, remove socket."
	socket
		ifNil: 
			[socket _ POPSocket new.
			
			Process fork: [socket serverName: Celeste popServer.
			socket userName: Celeste popUserName.
			socket password: self password.
			"socket addProgressObserver: Transcript."
			([socket connectToPOP]
				on: Error do: 
				[Transcript show: 'Couldn''t connect to POP.';
				 cr. (Delay forSeconds: 180) wait.
				false])
				ifTrue: 
					[messageCount _ socket numMessages.
					socket disconnectFromPOP.
					lastConnectTime _ Time now].
			socket _ nil]
				at: Processor userBackgroundPriority].
	^ messageCount! !

MessageTally makeSelector: #spyEvery:on: privilaged: false!
!MessageTally methodsFor: 'initialize-release' stamp: 'ls 7/11/2000 15:31'!
spyEvery: millisecs on: inputBlock 
	"Create a spy and spy on the given block at the specified rate."

	| myDelay value startTime time0 aBlock |
	aBlock := ObjectInspector realBlockFor: inputBlock.
	(aBlock isMemberOf: BlockContext)
		ifFalse: [self error: 'spy needs a block here'].
	self class: aBlock receiver class method: aBlock method.
		"set up the probe"
	ObservedProcess _ Processor activeProcess.
	myDelay := Delay forMilliseconds: millisecs.
	time0 := Time millisecondClockValue.
	Timer :=
		Process forBlock: [[true] whileTrue: 
			[startTime := Time millisecondClockValue.
			myDelay wait.
			self tally: ObservedProcess suspendedContext
				"tally can be > 1 if ran a long primitive"
				by: (Time millisecondClockValue - startTime) // millisecs].
		nil].
	Timer priority: Processor userInterruptPriority.
		"activate the probe and evaluate the block"
	Timer resume.
	value := aBlock value.
		"cancel the probe and return the value"
	Timer terminate.
	time := Time millisecondClockValue - time0.
	^value! !

ObjectInspector class makeSelector: #realBlockFor: privilaged: true!
!ObjectInspector class methodsFor: 'unwrapping' stamp: 'ls 8/4/2000 16:30'!
realBlockFor: aBlock
	| answer |
	"return the actual BlockContext underlying aBlock.  Does the right thing if aBlock is a BlockContext or a RestrictedBlock"
	answer := (self classOf: aBlock) == !!BlockContext
		ifTrue: [ aBlock ]
		ifFalse: [ self instVarOf: aBlock at: 1 ].
	answer ifNil: [ !!Error signal: 'found a RestrictedBlock wrapping nil!!' ].
	^answer! !

PWS class makeSelector: #serveOnPort:loggingTo: privilaged: false!
!PWS class methodsFor: 'Serving' stamp: 'ls 6/21/2000 13:42'!
serveOnPort: portNumber loggingTo: fileName
	"Start up the HTTP server loop for the given port number and log file."

	self stopServer.
	Socket initializeNetwork.
	ServerLog _ FileStream fileNamed: fileName.
	ServerLog position: ServerLog size.
	ServerPort _ ConnectionQueue portNumber: portNumber queueLength: 6.
	ClientNameCache _ Dictionary new.

	ServerProcess _ Process forBlock: [self loopOnPort: portNumber loggingTo: fileName].
	ServerProcess priority: Processor lowIOPriority.
	ServerProcess resume.
! !

PasteUpMorph makeSelector: #startBackgroundProcess privilaged: false!
!PasteUpMorph methodsFor: 'update cycle' stamp: 'ls 6/21/2000 13:42'!
startBackgroundProcess
	"Start a process to update this world in the background. Return the process created."

	| p |
	p _ Process forBlock: [[true] whileTrue: [
		self doOneCycleInBackground.
		(Delay forMilliseconds: 20) wait]].
	p resume.
	^ p
! !

PolygonMorphDashed class makeSelector: #animationExample privilaged: false!
!PolygonMorphDashed class methodsFor: 'as yet unclassified' stamp: 'ls 6/21/2000 13:36'!
animationExample
	"PolygonMorphDashed animationExample"
	| m numOfAnimationSteps start stop stepVec s delay delayAtAll point animation |
	m _ self
				vertices: {Display boundingBox origin. Display boundingBox corner}
				color: Color transparent
				firstBorderWidth: 2
				secondBorderWidth: 2
				firstBorderColor: Color black
				secondBorderColor: Color white
				firstBorderStepWidth: 10
				secondBorderStepWidth: 10.
	m lock.
	m openInWorld.
	m refreshWorld.
	(Delay forSeconds: 1) wait.
	numOfAnimationSteps _ 25.
	delayAtAll _ 1.
	start _ m vertices at: 1.
	stop _ m vertices at: 2.
	start = stop ifTrue: [^ self].
	stepVec _ stop - start / (numOfAnimationSteps + 1).
	s _ Array new: numOfAnimationSteps.
	1 to: numOfAnimationSteps do: 
		[:step | 
		point _ start + (step * stepVec).
		"avoid problems"
		s at: step put: (point = stop
				ifTrue: 
					[
					stop - (1 @ 1)]
				ifFalse: [point])].
	delay _ Delay forSeconds: delayAtAll / numOfAnimationSteps.
	animation _ 
			[1 to: s size do: 
				[:j | 
				"Move to the next point"
				m vertices at: 1 put: (s at: j).
				"update"
				m computeBounds.
				"not necessary here since forked"
				"	World refreshWorld."
				"for fast machines"
				delay wait].
			m delete].
	Process fork: animation! !

Process class makeSelector: #forBlock: privilaged: false!
!Process class methodsFor: 'instance creation' stamp: 'ls 6/20/2000 17:42'!
forBlock: aBlock 
	"Answer an instance of me that will execute aBlock at priority anInteger"

	^self forContext: 
			(ObjectInspector realBlockFor: [aBlock value.
				Processor terminateActive])
		priority: Processor activePriority! !
Process class makeSelector: #forBlock:priority: privilaged: false!
!Process class methodsFor: 'instance creation' stamp: 'ls 6/20/2000 17:41'!
forBlock: aBlock priority: anInteger 
	"Answer an instance of me that will execute aBlock at priority anInteger"

	^self forContext: 
			(ObjectInspector realBlockFor: [aBlock value.
				Processor terminateActive])
		priority: anInteger! !
Process class makeSelector: #fork: privilaged: false!
!Process class methodsFor: 'instance creation' stamp: 'ls 6/21/2000 13:34'!
fork: aBlock
	"Create a process executing aBlock and resume that process"
	^self fork: aBlock at: Processor activePriority! !
Process class makeSelector: #fork:at: privilaged: false!
!Process class methodsFor: 'instance creation' stamp: 'ls 6/21/2000 13:34'!
fork: aBlock  at: priority
	"Create a process executing aBlock and resume that process"
	| process |
	process := self forBlock: aBlock priority: priority.
	process resume.
	^process! !

ProcessorScheduler makeSelector: #yield privilaged: false!
!ProcessorScheduler methodsFor: 'process state change' stamp: 'ls 6/21/2000 13:36'!
yield
	"Give other Processes at the current priority a chance to run."

	| semaphore |
	semaphore _ Semaphore new.
	Process fork: [semaphore signal].
	semaphore wait! !

ProcessorScheduler class makeSelector: #startUp privilaged: false!
!ProcessorScheduler class methodsFor: 'background process' stamp: 'ls 6/21/2000 13:42'!
startUp
	"Install a background process of the lowest possible priority that is always runnable."
	"Details: The virtual machine requires that there is aways some runnable process that can be scheduled; this background process ensures that this is the case."

	Smalltalk installLowSpaceWatcher.
	BackgroundProcess == nil ifFalse: [BackgroundProcess terminate].
	BackgroundProcess _ Process forBlock: [self idleProcess].
	BackgroundProcess priority: SystemRockBottomPriority.
	BackgroundProcess resume.
! !

ProgressMorph class makeSelector: #example privilaged: false!
!ProgressMorph class methodsFor: 'example' stamp: 'ls 6/21/2000 13:36'!
example
	"ProgressMorph example"

	| progress |
	progress _ ProgressMorph label: 'Test progress'.
	progress subLabel: 'this is the subheading'.
	progress openInWorld.
	Process fork: [10 timesRepeat:
		[(Delay forMilliseconds: 200) wait.
		progress incrDone: 0.1].
	progress delete]! !

Project class makeSelector: #maybeForkInterrupt privilaged: false!
!Project class methodsFor: 'utilities' stamp: 'ls 6/21/2000 13:37'!
maybeForkInterrupt

	Preferences cmdDotEnabled ifFalse: [^self].
	Smalltalk isMorphic
		ifTrue: [Process fork: [self interruptName: 'User Interrupt']]
		ifFalse: [Process fork: [ScheduledControllers interruptName: 'User Interrupt']]! !
Project class makeSelector: #spawnNewProcess privilaged: false!
!Project class methodsFor: 'utilities' stamp: 'ls 6/21/2000 13:43'!
spawnNewProcess

	UIProcess _  Process forBlock:  [
		[World doOneCycle.  Processor yield.  false] whileFalse: [].
	]  priority: Processor userSchedulingPriority.
	UIProcess resume! !

RestrictedBlock makeSelector: #block: privilaged: false!
!RestrictedBlock methodsFor: 'initialization' stamp: 'ls 6/20/2000 13:32'!
block: aBlock
	block ifNotNil: [
		^self error: 'already initialized' ].
	block := aBlock! !
RestrictedBlock makeSelector: #class privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 8/4/2000 15:44'!
class
	^self weakClass! !
RestrictedBlock makeSelector: #ifError: privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/20/2000 17:24'!
ifError: errorHandlerBlock
	"Evaluate the block represented by the receiver. If an error occurs the given is evaluated with the error message and the receiver as parameters. The error handler block may return a value to be used if the receiver block gets an error. The receiver should not contain an explicit return statement as this would leave an obsolete error handler hanging around."
	"Note: it is more flexible to use #on:do: instead.  #on:do: allows trapping only specific errors, and it allows querying more details on what exactly happened"

	"Examples:
		[1 whatsUpDoc] ifError: [:err :rcvr | ^ 'huh?'].
		[1 / 0] ifError: [:err :rcvr |
			'ZeroDivide' = err
				ifTrue: [^ Float infinity]
				ifFalse: [self error: err]]
"

	^self on: Error do: [:ex |
		^errorHandlerBlock value: ex description value: ex receiver].


! !
RestrictedBlock makeSelector: #monitor privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/20/2000 17:24'!
monitor
	"Interactive. Morphic only.
	Make an UpdatingStringMorph that shows the result of evaluating me three times a second.
	Example: ([Smalltalk primBytesLeft] monitor)"

	Smalltalk isMorphic ifTrue: [
		Smalltalk at: #UpdatingStringMorph ifPresent: [:usm |
			^World currentHand attachMorph:
				((usm on: self selector: #value)
					stepTime: 333;
					openInWorld)]].
	self error: 'This only works in Morphic'! !
RestrictedBlock makeSelector: #timeToRun privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/20/2000 17:24'!
timeToRun
	"Answer the number of milliseconds taken to execute this block."

	^ Time millisecondsToRun: self
! !
RestrictedBlock makeSelector: #value privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/22/2000 11:34'!
value
	^block value! !
RestrictedBlock makeSelector: #value: privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/20/2000 13:32'!
value: anObject
	^block value: anObject! !
RestrictedBlock makeSelector: #value:value: privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/20/2000 13:33'!
value: anObject value: anotherObject
	^block value: anObject value: anotherObject! !
RestrictedBlock makeSelector: #value:value:value: privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/20/2000 13:33'!
value: firstObject value: secondObject value: thirdObject
	^block value: firstObject value: secondObject value: thirdObject
! !
RestrictedBlock makeSelector: #value:value:value:value: privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/20/2000 13:33'!
value: firstObject value: secondObject value: thirdObject value: fourthObject
	^block value: firstObject value: secondObject value: thirdObject value: fourthObject
! !
RestrictedBlock makeSelector: #valueWithArguments: privilaged: false!
!RestrictedBlock methodsFor: 'evaluating' stamp: 'ls 6/22/2000 11:34'!
valueWithArguments: arguments
	^block valueWithArguments: arguments
! !
RestrictedBlock makeSelector: #fixTemps privilaged: true!
!RestrictedBlock methodsFor: 'accessing' stamp: 'ls 8/11/2000 10:05'!
fixTemps
	^!!RestrictedBlock on: block fixTemps
! !
RestrictedBlock makeSelector: #numArgs privilaged: false!
!RestrictedBlock methodsFor: 'accessing' stamp: 'ls 6/20/2000 17:23'!
numArgs

	^block numArgs! !
RestrictedBlock makeSelector: #repeat privilaged: false!
!RestrictedBlock methodsFor: 'controlling' stamp: 'ls 6/20/2000 17:25'!
repeat
	"Evaluate the receiver repeatedly, ending only if the block explicitly returns."

	[self value. true] whileTrue! !
RestrictedBlock makeSelector: #repeatWithGCIf: privilaged: false!
!RestrictedBlock methodsFor: 'controlling' stamp: 'ls 6/20/2000 17:25'!
repeatWithGCIf: testBlock
	| ans |
	"run the receiver, and if testBlock returns true, garbage collect and run the receiver again"
	ans _ self value.
	(testBlock value: ans) ifTrue: [ Smalltalk garbageCollect. ans _ self value ].
	^ans! !
RestrictedBlock makeSelector: #whileFalse privilaged: false!
!RestrictedBlock methodsFor: 'controlling' stamp: 'ls 6/20/2000 17:25'!
whileFalse
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the receiver, as long as its value is false."
 
	^ [self value] whileFalse: []! !
RestrictedBlock makeSelector: #whileFalse: privilaged: false!
!RestrictedBlock methodsFor: 'controlling' stamp: 'ls 6/20/2000 17:25'!
whileFalse: aBlock 
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the argument, aBlock, as long as the value of the receiver is false."

	^ [self value] whileFalse: [aBlock value]! !
RestrictedBlock makeSelector: #whileTrue privilaged: false!
!RestrictedBlock methodsFor: 'controlling' stamp: 'ls 6/20/2000 17:25'!
whileTrue
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the receiver, as long as its value is true."
 
	^ [self value] whileTrue: []! !
RestrictedBlock makeSelector: #whileTrue: privilaged: false!
!RestrictedBlock methodsFor: 'controlling' stamp: 'ls 6/20/2000 17:25'!
whileTrue: aBlock 
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the argument, aBlock, as long as the value of the receiver is true."

	^ [self value] whileTrue: [aBlock value]! !
RestrictedBlock makeSelector: #printOn: privilaged: false!
!RestrictedBlock methodsFor: 'printing' stamp: 'ls 6/20/2000 17:26'!
printOn: aStream
	^block printString! !
RestrictedBlock makeSelector: #assert privilaged: false!
!RestrictedBlock methodsFor: 'exceptions' stamp: 'ls 6/20/2000 17:26'!
assert
	self assert: self! !
RestrictedBlock makeSelector: #ensure: privilaged: false!
!RestrictedBlock methodsFor: 'exceptions' stamp: 'ls 6/20/2000 17:27'!
ensure: aBlock
	"Evaluate a termination block after evaluating the receiver, regardless of whether the receiver's evaluation completes."
	^block ensure: aBlock! !
RestrictedBlock makeSelector: #ifCurtailed: privilaged: false!
!RestrictedBlock methodsFor: 'exceptions' stamp: 'ls 6/20/2000 17:27'!
ifCurtailed: aBlock
	"Evaluate the receiver with an abnormal termination action."
	^block ifCurtailed: aBlock! !
RestrictedBlock makeSelector: #on:do: privilaged: false!
!RestrictedBlock methodsFor: 'exceptions' stamp: 'ls 6/20/2000 17:27'!
on: exception do: handlerAction
	"Evaluate the receiver in the scope of an exception handler."
	^block  on: exception do: handlerAction! !
RestrictedBlock makeSelector: #notNil privilaged: false!
!RestrictedBlock methodsFor: 'testing' stamp: 'ls 6/22/2000 10:00'!
notNil
	^true! !
RestrictedBlock makeSelector: #clone privilaged: false!
!RestrictedBlock methodsFor: 'proxy restrictions' stamp: 'ls 8/4/2000 15:40'!
clone
	^self! !
RestrictedBlock makeSelector: #instVarAt: privilaged: false!
!RestrictedBlock methodsFor: 'proxy restrictions' stamp: 'ls 8/4/2000 15:41'!
instVarAt: index
	^self notAllowed! !
RestrictedBlock makeSelector: #instVarAt:put: privilaged: false!
!RestrictedBlock methodsFor: 'proxy restrictions' stamp: 'ls 8/4/2000 15:41'!
instVarAt: index put: value
	^self notAllowed! !
RestrictedBlock makeSelector: #shallowCopy privilaged: false!
!RestrictedBlock methodsFor: 'proxy restrictions' stamp: 'ls 8/4/2000 15:41'!
shallowCopy	
	^self! !
RestrictedBlock makeSelector: #copy privilaged: true!
!RestrictedBlock methodsFor: 'misc' stamp: 'ls 8/11/2000 10:05'!
copy
	^!!RestrictedBlock on: block copy
! !

RestrictedBlock class makeSelector: #on: privilaged: false!
!RestrictedBlock class methodsFor: 'instance creation' stamp: 'ls 6/20/2000 17:28'!
on: realBlock
	^super new block: realBlock; yourself! !

Scamper makeSelector: #jumpToAbsoluteUrl: privilaged: false!
!Scamper methodsFor: 'changing page' stamp: 'ls 6/21/2000 13:43'!
jumpToAbsoluteUrl: urlText
	"start downloading a new page.  The page source is downloaded in a background thread"
	|  newUrl newSource |

	self stopEverything.

	"get the new url"
	newUrl _ urlText asUrl.


	"if it fundamentally doesn't fit the pages-and-contents model used internally, spawn off an external viewer for it"
	newUrl hasContents ifFalse: [ newUrl activate.  ^true ].

	"fork a Process to do the actual downloading, parsing, and formatting.  It's results will be picked up in #step"
	self status: 'downloading ', newUrl toText, '...'.

	downloadingProcess _ Process forBlock: [ 
	  	newSource _ [ newUrl retrieveContentsForBrowser: self ] ifError: [ :msg :ctx |
			MIMEDocument contentType: 'text/plain' content: msg ].

		newSource 
			ifNil: [ newSource _ MIMEDocument contentType: 'text/plain' content: 'Error retrieving this URL' ].

			newSource url ifNil: [
				newSource _ MIMEDocument contentType: newSource contentType  content: newSource content  url: newUrl ].

			documentQueue nextPut: newSource.
			downloadingProcess _ nil.
	].

	downloadingProcess resume.

	[recentDocuments size > currentUrlIndex] whileTrue: [
		"delete all elements in recentDocuments after currentUrlIndex"
		recentDocuments removeLast.
	].
	currentUrlIndex _ currentUrlIndex + 1.

	^true! !
Scamper makeSelector: #startDownloadingMorphState: privilaged: false!
!Scamper methodsFor: 'changing page' stamp: 'ls 6/21/2000 13:43'!
startDownloadingMorphState: morphs
	downloadingProcess _ Process forBlock: [
		morphs do: [ :m | m downloadStateIn: self].
	].
	downloadingProcess resume.! !
Scamper makeSelector: #startDownloadingStateIn:url: privilaged: false!
!Scamper methodsFor: 'changing page' stamp: 'ls 6/21/2000 13:43'!
startDownloadingStateIn: aDocument  url: aUrl
	"download the state for the given document in a background thread.  signal the foreground when the data has arrived"
	downloadingProcess _ Process forBlock: [	
		aDocument allSubentitiesDo: [ :e |
			e downloadState: aUrl ].
		documentQueue nextPut: #stateDownloaded.
		downloadingProcess _ nil. ].
	downloadingProcess resume.! !
Scamper makeSelector: #submitFormWithInputs:url:method: privilaged: false!
!Scamper methodsFor: 'changing page' stamp: 'ls 6/21/2000 13:43'!
submitFormWithInputs: inputs  url: url  method: method
	| newUrl newSource | 
	self stopEverything.

	(method asLowercase ~= 'get' and: [ method asLowercase ~= 'post' ]) ifTrue: [
		self notify: 'unkown FORM method: ', method.
		^false ].

	newUrl _ url asUrlRelativeTo: currentUrl.	

	newUrl schemeName ~= 'http' ifTrue: [
		self notify: 'I can only submit forms via HTTP'.
		^false ].

	self status: 'submitting form...'.

	downloadingProcess _ Process forBlock: [
			method asLowercase = 'get' 
				ifTrue: [newSource _ newUrl retrieveContentsArgs: inputs] 
				ifFalse: [newSource _ newUrl postFormArgs: inputs].
			documentQueue nextPut:  newSource.

			downloadingProcess _ nil.
		].

		downloadingProcess resume.
		^true
! !
Scamper makeSelector: #submitFormWithInputs:url:method:encoding: privilaged: false!
!Scamper methodsFor: 'changing page' stamp: 'ls 6/21/2000 13:43'!
submitFormWithInputs: inputs url: url method: method encoding: encoding
	| newUrl newSource | 
	self stopEverything.

	(method asLowercase ~= 'get' and: [ method asLowercase ~= 'post' ]) ifTrue: [
		self notify: 'unkown FORM method: ', method.
		^false ].

	newUrl _ url asUrlRelativeTo: currentUrl.	

	newUrl schemeName ~= 'http' ifTrue: [
		self notify: 'I can only submit forms via HTTP'.
		^false ].

	self status: 'submitting form...'.

	downloadingProcess _ Process forBlock: [
		method asLowercase = 'get' 
			ifTrue: [newSource _ newUrl retrieveContentsArgs: inputs] 
			ifFalse: [
				encoding = MIMEDocument contentTypeMultipart
					ifTrue: [newSource _ newUrl postMultipartFormArgs: inputs]
					ifFalse: [newSource _ newUrl postFormArgs: inputs]].
		documentQueue nextPut:  newSource.

		downloadingProcess _ nil.
	].

	downloadingProcess resume.
	^true
! !

ScorePlayer makeSelector: #startMIDIPlaying privilaged: false!
!ScorePlayer methodsFor: 'midi output' stamp: 'ls 6/21/2000 13:43'!
startMIDIPlaying
	"Start up a process to play this score via MIDI."

	midiPort ensureOpen.
	midiPlayerProcess ifNotNil: [midiPlayerProcess terminate].
	midiPlayerProcess _ Process forBlock: [self midiPlayLoop].
	midiPlayerProcess
		priority: Processor userInterruptPriority;
		resume.
! !

SimpleClientSocket class makeSelector: #forkingRemoteCursorSender privilaged: false!
!SimpleClientSocket class methodsFor: 'remote cursor example' stamp: 'ls 6/21/2000 13:37'!
forkingRemoteCursorSender
	"This is the client side of a test that sends samples of the local input sensor state to the server, which may be running on a local or remote host. This method opens the connection, then forks a process to send the cursor data. Data is sent continuously until the user clicks in a 20x20 pixel square at the top-left corner of the display. The server should be started first. Note the server's address, since this method will prompt you for it."
	"SimpleClientSocket forkingRemoteCursorSender"

	| sock addr stopRect |
	Transcript show: 'starting remote cursor sender'; cr.
	Transcript show: 'initializing network'; cr.
	Socket initializeNetwork.
	addr _ NetNameResolver promptUserForHostAddress.
	Transcript show: 'opening connection'; cr.
	sock _ SimpleClientSocket new.
	sock connectTo: addr port: 54323.
	sock waitForConnectionUntil: self standardDeadline.
	(sock isConnected) ifFalse: [self error: 'sock not connected'].
	Transcript show: 'connection established'; cr.

	stopRect _ 0@0 corner: 20@20.  "click in this rectangle to stop sending"
	Display reverse: stopRect.
	Process fork: ["the sending process"
		[(stopRect containsPoint: Sensor cursorPoint) and:
		 [Sensor anyButtonPressed]]
			whileFalse: [
				sock sendCommand: self sensorStateString.
				(Delay forMilliseconds: 20) wait].

		sock waitForSendDoneUntil: self standardDeadline.
		sock destroy.
		Transcript show: 'remote cursor sender done'; cr.
		Display reverse: stopRect.
	].
! !

SoundPlayer class makeSelector: #startPlayerProcessBufferSize:rate:stereo: privilaged: false!
!SoundPlayer class methodsFor: 'player process' stamp: 'ls 6/21/2000 13:43'!
startPlayerProcessBufferSize: bufferSize rate: samplesPerSecond stereo: stereoFlag
	"Start the sound player process. Terminate the old process, if any."
	"SoundPlayer startPlayerProcessBufferSize: 1000 rate: 11025 stereo: false"

	self stopPlayerProcess.
	ActiveSounds _ OrderedCollection new.
	Buffer _ SoundBuffer newStereoSampleCount: (bufferSize // 4) * 4.
	PlayerSemaphore _ Semaphore forMutualExclusion.
	SamplingRate _ samplesPerSecond.
	Stereo _ stereoFlag.
	ReadyForBuffer _ Semaphore new.
	SoundSupported _ true. "Assume so"
	UseReadySemaphore _ true.  "set to false if ready semaphore not supported by VM"
	self primSoundStartBufferSize: Buffer stereoSampleCount
		rate: samplesPerSecond
		stereo: Stereo
		semaIndex: (Smalltalk registerExternalObject: ReadyForBuffer).
	"Check if sound start prim was successful"
	SoundSupported ifFalse:[^self].
	UseReadySemaphore
		ifTrue: [PlayerProcess _ Process forBlock: [SoundPlayer playLoop]]
		ifFalse: [PlayerProcess _ Process forBlock: [SoundPlayer oldStylePlayLoop] ].
	UseReverb ifTrue: [self startReverb].

	PlayerProcess priority: Processor userInterruptPriority.
	PlayerProcess resume.! !

SoundRecorder makeSelector: #startRecording privilaged: false!
!SoundRecorder methodsFor: 'recording controls' stamp: 'ls 6/21/2000 13:43'!
startRecording
	"Turn of the sound input driver and start the recording process. Initially, recording is paused."

	| semaIndex |
	recordLevel ifNil: [recordLevel _ 0.5].  "lazy initialization"
	CanRecordWhilePlaying ifFalse: [SoundPlayer shutDown].
	recordProcess ifNotNil: [self stopRecording].
	paused _ true.
	meteringBuffer _ SoundBuffer newMonoSampleCount: 1024.
	meterLevel _ 0.
	self allocateBuffer.
	bufferAvailableSema _ Semaphore new.
	semaIndex _ Smalltalk registerExternalObject: bufferAvailableSema.
	self primStartRecordingDesiredSampleRate: samplingRate asInteger
		stereo: stereo
		semaIndex: semaIndex.
	samplingRate _ self primGetActualRecordingSampleRate.
	self primSetRecordLevel: (1000.0 * recordLevel) asInteger.
	recordProcess _ Process forBlock:  [self recordLoop].
	recordProcess priority: Processor userInterruptPriority.
	recordProcess resume.
! !

SystemDictionary makeSelector: #installLowSpaceWatcher privilaged: false!
!SystemDictionary methodsFor: 'memory space' stamp: 'ls 6/21/2000 13:43'!
installLowSpaceWatcher
	"Start a process to watch for low-space conditions."
	"Smalltalk installLowSpaceWatcher"

	self primSignalAtBytesLeft: 0.  "disable low-space interrupts"
	LowSpaceProcess == nil ifFalse: [LowSpaceProcess terminate].
	LowSpaceProcess _ Process forBlock: [self lowSpaceWatcher].
	LowSpaceProcess priority: Processor lowIOPriority.
	LowSpaceProcess resume.

! !
SystemDictionary makeSelector: #handleUserInterrupt privilaged: false!
!SystemDictionary methodsFor: 'miscellaneous' stamp: 'ls 6/21/2000 13:37'!
handleUserInterrupt
	Preferences cmdDotEnabled ifTrue:
		[Smalltalk isMorphic
			ifTrue: [Process fork: [CurrentProjectRefactoring currentInterruptName: 'User Interrupt']]
			ifFalse: [Process fork: [ScheduledControllers interruptName: 'User Interrupt']]]! !

SystemMonitor class makeSelector: #start: privilaged: false!
!SystemMonitor class methodsFor: 'scheduling' stamp: 'ls 6/21/2000 13:43'!
start: aMonitor
	self stop.
	ActiveMonitor _ aMonitor.
	ActiveClock _
		Process forBlock: [[true] whileTrue:
			[ActiveMonitor display.
			(Delay forMilliseconds: MonitorDelay) wait]].
	ActiveClock priority: Processor lowIOPriority.
	ActiveClock resume! !

Time class makeSelector: #benchmarkPrimitiveResponseDelay privilaged: false!
!Time class methodsFor: 'benchmarks' stamp: 'ls 6/21/2000 13:40'!
benchmarkPrimitiveResponseDelay	"Time benchmarkPrimitiveResponseDelay"
	"Benchmark the overhead for primitive dispatches with an active Delay.
	On the VM level, this tests the efficiency of ioLowResMSecs."

	"PII/400 Windows98: 0.128 microseconds per prim"

	"ar 9/6/1999: This value is *extremely* important for stuff like sockets etc.
	I had a bad surprise when Michael pointed this particular problem out:
	Using the hardcoded clock() call for ioLowResMSecs on Win32 resulted in an overhead
	of 157.4 microseconds per primitive call - meaning you can't get no more than
	approx. 6000 primitives per second on my 400Mhz PII system with an active delay!!
	BTW, it finally explains why Squeak seemed soooo slow when running PWS or 
	other socket stuff. The new version (not using clock() but some Windows function) 
	looks a lot better (see above; approx. 8,000,000 prims per sec with an active delay)."

	| nLoops bb index baseTime actualTime delayTime |
	delayTime _ 5000. "Time to run this test is approx. 3*delayTime"

	Delay anyActive ifTrue:[
		^self notify:'Some delay is currently active.
Running this benchmark will not give any useful result.'].

	bb _ Array new: 1. "The object we send the prim message to"

	"Compute the # of loops we'll run in a decent amount of time"
	Process fork: [(Delay forMilliseconds: delayTime) wait] 
		at: Processor userInterruptPriority.

	nLoops _ 0.
	[Delay anyActive] whileTrue:[
		bb basicSize; basicSize; basicSize; basicSize; basicSize; 
			basicSize; basicSize; basicSize; basicSize; basicSize.
		nLoops _ nLoops + 1.
	].

	"Flush the cache and make sure #basicSize is in there"
	Object flushCache.
	bb basicSize.

	"Now run the loop without any active delay
	for getting an idea about its actual speed."
	baseTime _ self millisecondClockValue.
	index _ nLoops.
	[index > 0] whileTrue:[
		bb basicSize; basicSize; basicSize; basicSize; basicSize; 
			basicSize; basicSize; basicSize; basicSize; basicSize.
		index _ index - 1.
	].
	baseTime _ self millisecondClockValue - baseTime.

	"Setup the active delay but try to never make it active"
	Process fork: [(Delay forMilliseconds: delayTime + delayTime) wait] 
		at: Processor userInterruptPriority.

	"And run the loop"
	actualTime _ self millisecondClockValue.
	index _ nLoops.
	[index > 0] whileTrue:[
		bb basicSize; basicSize; basicSize; basicSize; basicSize; 
			basicSize; basicSize; basicSize; basicSize; basicSize.
		index _ index - 1.
	].
	actualTime _ self millisecondClockValue - actualTime.

	"And get us some result"
	^((actualTime - baseTime) * 1000 asFloat / (nLoops * 10) truncateTo: 0.001) printString,
		' microseconds overhead per primitive call'! !

Transcripter class makeSelector: #startTranscriptProcess privilaged: false!
!Transcripter class methodsFor: 'instance creation' stamp: 'ls 6/21/2000 13:43'!
startTranscriptProcess   "Transcripter startTranscriptProcess"
	| activeProcess |
	Transcript _ self newInFrame: Display boundingBox.
	activeProcess _ Process forBlock:  [Transcript readEvalPrint.
					Smalltalk processShutDownList: true; quitPrimitive]
					priority: Processor userSchedulingPriority.
	activeProcess resume.
	Processor terminateActive
! !

Utilities class makeSelector: #retrieveUrls:ontoQueue:withWaitSema: privilaged: false!
!Utilities class methodsFor: 'fetching updates' stamp: 'ls 6/21/2000 13:44'!
retrieveUrls: urls  ontoQueue: queue  withWaitSema: waitSema
	| doc |
	"download the given list of URLs.  The queue will be loaded alternately with url's and with the retrieved contents.  If a download fails, the contents will be #failed.  If all goes well, a special pair with an empty URL and the contents #finished will be put on the queue.  waitSema is waited on every time before a new document is downloaded; this keeps the downloader from getting too far ahead of the main process"

	"kill the existing downloader if there is one"
	UpdateDownloader ifNotNil: [ UpdateDownloader terminate ].

	"fork a new downloading process"
	UpdateDownloader :=  Process forBlock: [ urls do: [ :url |
		waitSema wait.
		queue nextPut: url.
		doc _ HTTPSocket httpGet: url accept: 'application/octet-stream'.
		doc class == String
			ifTrue: [ queue nextPut: #failed.  Processor activeProcess terminate ]
			ifFalse: [ queue nextPut: doc] ] .

		queue nextPut: ''.
		queue nextPut: #finished.
	].
	UpdateDownloader priority: Processor userInterruptPriority.


	"start the process running"
	UpdateDownloader resume.
! !

WeakArray class makeSelector: #initialize privilaged: false!
!WeakArray class methodsFor: 'class initialization' stamp: 'ls 6/21/2000 13:44'!
initialize
	"WeakArray initialize"
	"Do we need to initialize specialObjectsArray?"
	Smalltalk specialObjectsArray size < 42 
		ifTrue:[Smalltalk recreateSpecialObjectsArray].
	"Check if Finalization is supported by this VM"
	IsFinalizationSupported _ nil.
	self isFinalizationSupported ifFalse:[^self].

	FinalizationProcess notNil ifTrue:[FinalizationProcess terminate].
	FinalizationSemaphore := Smalltalk specialObjectsArray at: 42.
	FinalizationDependents isNil ifTrue:[
		FinalizationDependents := WeakArray new: 10.
	].
	FinalizationLock := Semaphore forMutualExclusion.
	FinalizationProcess := Process forBlock: [self finalizationProcess].
	FinalizationProcess priority: Processor userInterruptPriority.
	FinalizationProcess resume.! !

WorldState makeSelector: #startBackgroundProcess privilaged: false!
!WorldState methodsFor: 'update cycle' stamp: 'ls 6/21/2000 13:44'!
startBackgroundProcess
	"Start a process to update this world in the background. Return the process created."

	| p |
	p _ Process forBlock: [[true] whileTrue: [
		self doOneCycleInBackground.
		(Delay forMilliseconds: 20) wait]].
	p resume.
	^ p
! !


!EventSensor methodsFor: 'initialize' stamp: 'ls 8/21/2000 20:40'!
startUp
	"Run the I/O process"
	self shutDown.
	self initialize.
	self primSetInputSemaphore: (Smalltalk registerExternalObject: inputSemaphore).
	inputProcess _ Process fork: [self ioProcess] at: Processor lowIOPriority.
	super startUp.! !

WeakArray initialize!
UndefinedObject removeSelector: #DoIt!

!RestrictedBlock reorganize!
('initialization' block:)
('evaluating' class ifError: monitor timeToRun value value: value:value: value:value:value: value:value:value:value: valueWithArguments:)
('accessing' fixTemps numArgs)
('controlling' repeat repeatWithGCIf: whileFalse whileFalse: whileTrue whileTrue:)
('printing' printOn:)
('exceptions' assert ensure: ifCurtailed: on:do:)
('testing' notNil)
('proxy restrictions' clone instVarAt: instVarAt:put: shallowCopy)
('misc' copy)
!

BlockContext removeSelector: #fork!
BlockContext removeSelector: #forkAt:!
BlockContext removeSelector: #newProcess!
BlockContext removeSelector: #newProcessWith:!
