'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 11:59:32 am'!
"Change Set:		IslandsBase
Date:			26 June 2000
Author:			Lex Spoon

Defines the Island structure, and adds methods to set and retrieve the currently active island"!

Object subclass: #Island
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sandbox-Islands'!

!Island commentStamp: '<historical>' prior: 0!
An environment for code development and execution.  It has its own group of globals, classes, and pools.!
Island subclass: #IsolatedIsland
	instanceVariableNames: 'classProxies pools globalPool classes '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sandbox-Islands'!
Link subclass: #Process
	instanceVariableNames: 'suspendedContext priority myList island '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!
Island subclass: #SystemIsland
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sandbox-Islands'!

!SystemIsland commentStamp: '<historical>' prior: 0!
An island which reflects the standard Squeak system as a whole.!
Behavior makeSelector: #metaclass privilaged: true!
!Behavior methodsFor: 'accessing' stamp: 'ls 5/25/2000 14:48'!
metaclass
	<primitive: 111>
	self primitiveFailed! !

Debugger class makeSelector: #openInterrupt:onProcess: privilaged: false!
!Debugger class methodsFor: 'opening' stamp: 'ls 8/7/2000 18:03'!
openInterrupt: aString onProcess: interruptedProcess
	"Open a notifier in response to an interrupt. An interrupt occurs when the user types the interrupt key (cmd-. on Macs, ctrl-c or alt-. on other systems) or when the low-space watcher detects that memory is low."
	| debugger |
	self simulationGuard.

	debugger _ self new.
	debugger
		process: interruptedProcess
		controller: ((Smalltalk isMorphic not
					and: [ScheduledControllers activeControllerProcess == interruptedProcess])
						ifTrue: [ScheduledControllers activeController])
		context: interruptedProcess suspendedContext.
	debugger externalInterrupt: true.

Preferences logDebuggerStackToFile ifTrue:
	[(aString includesSubString: 'Space') & 
		(aString includesSubString: 'low') ifTrue: [
			Smalltalk logError: aString inContext: debugger interruptedContext to:'LowSpaceDebug.log']].

	^ debugger
		openNotifierContents: debugger interruptedContext shortStack
		label: aString
! !

Island makeSelector: #associationForVariable:inPool:ifAbsent: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 7/6/2000 16:34'!
associationForVariable: variableName  inPool: poolName  ifAbsent: errorBlock
	"look up a pool variable in the given pool name.  If poolName is nil, look in the global table.  Returns the binding used to implement that variable"
	^self subclassResponsibility! !
Island makeSelector: #classNamed:ifAbsent: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 5/25/2000 12:40'!
classNamed: aSymbol ifAbsent: errorBlock
	"return the class with the given name"
	^self subclassResponsibility! !
Island makeSelector: #classProxyFor: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 5/25/2000 12:39'!
classProxyFor: realClass
	"return the class proxy to be used for the given real class"
	^self subclassResponsibility! !
Island makeSelector: #literalWithCanonicalValue: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 5/25/2000 13:08'!
literalWithCanonicalValue: value
	"return the literal who has the given canonical value.  Each space will have its own copy of the literal"
	^self subclassResponsibility! !
Island makeSelector: #loadSerializedObject: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 6/8/2000 10:49'!
loadSerializedObject: aString
        "load an object from aString, using classes inside the island"
        ^self notYetImplemented! !
Island makeSelector: #metaclassNamed:ifAbsent: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 5/25/2000 14:46'!
metaclassNamed: aSymbol ifAbsent: errorBlock
	"return the metaclass with the given name"
	^(self classNamed: aSymbol ifAbsent: [ ^errorBlock value]) metaclass! !
Island makeSelector: #setVariable:toValue:inPool: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 6/8/2000 10:57'!
setVariable: variableName toValue: value inPool: poolName
        "set a pool variable.  If poolName is nil, look in the global table"
        ^self setVariable: variableName toValue: value inPool: poolName ifAbsent: [ self error: 'variable not present' ]. ! !
Island makeSelector: #setVariable:toValue:inPool:ifAbsent: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 5/25/2000 13:11'!
setVariable: variableName toValue: value inPool: poolName  ifAbsent: errorBlock
	"set a pool variable.  If poolName is nil, look in the global table"
	^self subclassResponsibility! !
Island makeSelector: #valueOfVariable:inPool: privilaged: true!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 7/5/2000 15:19'!
valueOfVariable: variableName  inPool: poolName
	"look up a pool variable in the given pool name.  If poolName is nil, look in the global table"
	^self valueOfVariable: variableName inPool: poolName ifAbsent: [nil].
! !
Island makeSelector: #valueOfVariable:inPool:ifAbsent: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 7/6/2000 16:36'!
valueOfVariable: variableName  inPool: poolName  ifAbsent: errorBlock
	"look up a pool variable in the given pool name.  If poolName is nil, look in the global table"
	| binding |
	binding := self associationForVariable: variableName  inPool: poolName  ifAbsent: [^errorBlock value ].
	^binding value! !
Island makeSelector: #weakClassProxyFor: privilaged: false!
!Island methodsFor: 'as yet unclassified' stamp: 'ls 6/20/2000 16:45'!
weakClassProxyFor: realClass
	"return the class proxy to be used for the given real class, assuming realClass is so dangerous it should not allow being instantiated"
	^self classProxyFor: realClass! !

IsolatedIsland makeSelector: #addClassPoolFor: privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 8/12/2000 13:23'!
addClassPoolFor: class
	"Add a class pool for the given class.  Just sets all hte variables to nil"
	| table |
	class classPool isEmpty ifTrue: [ ^self ].
	table := Dictionary new.
	class classPool keysDo: [ :key | table at: key put: nil ].
	pools at: class name put: table ! !
IsolatedIsland makeSelector: #addClassesWithSafeSubclasses: privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 6/8/2000 10:40'!
addClassesWithSafeSubclasses: safeClasses
        "copy in all classes where neither the class nor its metaclass defines a privilaged method.  assume the given l
ist of classes is safe"
        safeClasses do: [ :class |
                self addClassWithSafeSubclasses: class withOtherSafeClasses: safeClasses ].
! !
IsolatedIsland makeSelector: #addPool:named: privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 8/11/2000 16:50'!
addPool: pool named: name
	pools at: name put: pool! !
IsolatedIsland makeSelector: #addSafeClasses privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 6/8/2000 10:34'!
addSafeClasses
        "copy in all classes where neither the class nor its metaclass defines a privilaged method"
        Smalltalk allClassesDo: [ :class |
                (class definesPrivilagedMethod not and: [ class class definesPrivilagedMethod not]) ifTrue: [
                        globalPool at: class name put: class ] ].! !
IsolatedIsland makeSelector: #addSafeClassesUnder: privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 6/8/2000 10:41'!
addSafeClassesUnder: safeClasses
        "copy in subclasses of the given classes where neither the class nor its metaclass defines a privilaged method.
  Also adds the given list of classes"
        safeClasses do: [ :class |
                self addSafeClassesUnder: class withOtherSafeClasses: safeClasses ].
! !
IsolatedIsland makeSelector: #addSafeClassesUnder:withOtherSafeClasses: privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 6/9/2000 11:25'!
addSafeClassesUnder: aClass withOtherSafeClasses: otherSafeClasses
        "add class, and then add all subclasses which either have no privilaged methods, or which are in otherSafeClass
es"
        globalPool at: aClass name put: aClass.
        pools at: aClass name put: aClass classPool.
        aClass subclasses do: [ :subclass |
                subclass isMeta ifFalse: [
                        (subclass definesPrivilagedMethod not and: [ subclass class definesPrivilagedMethod not ]) ifTrue: [
                                self addSafeClassesUnder: subclass withOtherSafeClasses: otherSafeClasses ] ] ].! !
IsolatedIsland makeSelector: #deepCopyPool: privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 6/8/2000 10:47'!
deepCopyPool: poolName
        | newTable |
        "copy the given pool and its contents into the local space.  does not install the pool under a global name..."
        newTable := Dictionary new.
        pools at: poolName put: newTable.
        (Smalltalk at: poolName) keysAndValuesDo: [ :key :value |
                newTable at: key deepCopy put: value deepCopy ].! !
IsolatedIsland makeSelector: #initialize privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 6/8/2000 10:32'!
initialize
        classProxies := IdentityDictionary new.
        pools := Dictionary new.
        globalPool := Dictionary new.! !
IsolatedIsland makeSelector: #removeVariable:fromPool: privilaged: false!
!IsolatedIsland methodsFor: 'setup' stamp: 'ls 8/11/2000 17:09'!
removeVariable: name fromPool: poolName
	"remove the given variable from the given pool"
	| pool |
	pool := poolName ifNil: [ globalPool ] ifNotNil: [ pools at: poolName ].
	pool removeKey: name ifAbsent: [].! !
IsolatedIsland makeSelector: #associationForVariable:inPool:ifAbsent: privilaged: false!
!IsolatedIsland methodsFor: 'lookups' stamp: 'ls 7/12/2000 15:32'!
associationForVariable: variableName  inPool: poolName  ifAbsent: errorBlock
        | pool |
        poolName
                ifNil: [ pool := globalPool ]
                ifNotNil: [ pool := pools at: poolName ifAbsent: [^errorBlock value] ].

        ^pool associationAt: variableName ifAbsent: errorBlock
! !
IsolatedIsland makeSelector: #classProxyFor: privilaged: true!
!IsolatedIsland methodsFor: 'lookups' stamp: 'ls 8/11/2000 10:05'!
classProxyFor: realClass
        ^classProxies at: realClass ifAbsentPut: [ !!RestrictedClass on: realClass ]
! !
IsolatedIsland makeSelector: #setVariable:toValue:inPool: privilaged: false!
!IsolatedIsland methodsFor: 'lookups' stamp: 'ls 6/8/2000 11:03'!
setVariable: variableName toValue: value inPool: poolName
        | pool |
        poolName
                ifNil: [ pool := globalPool ]
                ifNotNil: [ pool := pools at: poolName ifAbsentPut: [ Dictionary new] ].

        pool at: variableName put: value.
        ^value! !
IsolatedIsland makeSelector: #weakClassProxyFor: privilaged: true!
!IsolatedIsland methodsFor: 'lookups' stamp: 'ls 8/14/2000 15:08'!
weakClassProxyFor: realClass
	"return the class proxy to be used for the given real class, assuming realClass is so dangerous it should not allow being instantiated"
	^!!ReallyRestrictedClass on: realClass! !

Process makeSelector: #island privilaged: true!
!Process methodsFor: 'islands' stamp: 'ls 8/4/2000 16:33'!
island
	"return the island this process is currently using"
	self flag: #xxx.  "the auto-initialization should be removed eventually, but it's handy during development of the system.  new processes should probably inherit their parent's island"
	island ifNil: [ island := !!SystemIsland new ].
	^island
! !
Process makeSelector: #setIsland: privilaged: false!
!Process methodsFor: 'islands' stamp: 'ls 5/25/2000 12:46'!
setIsland: newIsland
	"change the current island.  Mainly used for initialization; setIsland:during: is more commonly used"
	island := newIsland! !
Process makeSelector: #setIsland:during: privilaged: true!
!Process methodsFor: 'islands' stamp: 'ls 8/15/2000 20:10'!
setIsland: newIsland  during: aBlock
	"change the current island during execution of aBlock"
	| savedIsland exception stack |
	savedIsland := island.
	island := newIsland.
	aBlock on: !!Exception do: [ :ex | 
		(!!ExceptionAboutToReturn handles: ex) ifTrue: [ ex pass ].
		exception := ex.  island := savedIsland.
		stack := (!!ObjectInspector instVarOf: exception at: 2) longStack. island := newIsland. ].
	island := savedIsland.
	exception ifNotNil: [ self error: 'exception happened' ].
! !

ProcessorScheduler makeSelector: #doInSystemIsland: privilaged: false!
!ProcessorScheduler methodsFor: 'islands' stamp: 'ls 8/14/2000 14:33'!
doInSystemIsland: aBlock
	"perform aBlock in a SystemIsland"
	self activeProcess setIsland: nil during: aBlock! !

SystemDictionary makeSelector: #currentIsland privilaged: true!
!SystemDictionary methodsFor: 'islands' stamp: 'ls 8/4/2000 16:33'!
currentIsland
	"return the Island that is currently active in this process"
	^!!Processor activeProcess island! !

SystemIsland makeSelector: #associationForVariable:inPool:ifAbsent: privilaged: true!
!SystemIsland methodsFor: 'as yet unclassified' stamp: 'ls 8/4/2000 16:34'!
associationForVariable: variableName  inPool: poolName  ifAbsent: errorBlock
	| pool |
	poolName
		ifNil: [ pool := !!Smalltalk ]
		ifNotNil: [ pool := !!Smalltalk at: poolName ifAbsent: [ ^errorBlock value ] ].

	pool isBehavior ifTrue: [ pool := pool classPool ].

	^pool associationAt: variableName ifAbsent: errorBlock
! !
SystemIsland makeSelector: #classNamed:ifAbsent: privilaged: true!
!SystemIsland methodsFor: 'as yet unclassified' stamp: 'ls 8/4/2000 16:34'!
classNamed: aSymbol ifAbsent: errorBlock
	^!!Smalltalk at: aSymbol ifAbsent: errorBlock! !
SystemIsland makeSelector: #classProxyFor: privilaged: false!
!SystemIsland methodsFor: 'as yet unclassified' stamp: 'ls 5/25/2000 13:08'!
classProxyFor: realClass
	"use the real class"
	^realClass! !
SystemIsland makeSelector: #setVariable:toValue:inPool:ifAbsent: privilaged: true!
!SystemIsland methodsFor: 'as yet unclassified' stamp: 'ls 8/4/2000 16:34'!
setVariable: variableName toValue: value inPool: poolName  ifAbsent: errorBlock
	| pool |
	poolName
		ifNil: [ pool := !!Smalltalk ]
		ifNotNil: [ pool := !!Smalltalk at: poolName ifAbsent: [ ^errorBlock value ] ].

	pool isBehavior ifTrue: [ "for looking in class pools" pool := pool classPool ].
	(pool includesKey: variableName) ifFalse: [ ^errorBlock value ].
	pool at: variableName put: value.

	^value! !


!ProcessorScheduler reorganize!
('accessing' activePriority activeProcess highestPriority highestPriority:)
('removing' remove:ifAbsent:)
('process state change' suspendFirstAt: suspendFirstAt:ifNone: terminateActive yield)
('priority names' highIOPriority lowIOPriority systemBackgroundPriority timingPriority userBackgroundPriority userInterruptPriority userSchedulingPriority)
('private' anyProcessesAbove:)
('objects from disk' objectForDataStream:)
('islands' doInSystemIsland:)
!

