'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 11:59:57 am'!
"Change Set:		PrivilagedMethods
Date:			13 June 2000
Author:			Lex Spoon

Adds a mechanism for tagging selectors as privilaged or unprivilaged.  A privilaged method is allowed to access all features the compiler allows, while an unprivilaged method is more restricted"!

Behavior subclass: #ClassDescription
	instanceVariableNames: 'instanceVariables organization fullyPrivilagedSelectors '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Classes'!

!Browser methodsFor: 'message functions' stamp: 'ls 6/5/2000 15:09'!
messageListMenu: aMenu shifted: shifted
	^ shifted ifFalse: [aMenu labels:
'browse full (b)
fileOut
printOut
senders of...
implementors of... (m)
smart senders (n)
method inheritance
versions (v)
toggle privilage
inst var refs...
inst var defs...
class var refs...
class variables
class refs (N)
remove
more...'
	lines: #(3 9 14)
	selections:
		#(browseMethodFull fileOutMessage printOutMessage
		browseSendersOfMessages browseMessages smartBrowseSenders methodHierarchy browseVersions toggleMessagePrivilage
		browseInstVarRefs browseInstVarDefs browseClassVarRefs 
			browseClassVariables browseClassRefs
		removeMessage shiftedYellowButtonActivity )]

	ifTrue: [aMenu labels: 'browse class hierarchy
browse class
browse method (O)
make stand-alone method pane
toggle diffing
implementors of sent messages
change sets with this method
inspect instances
inspect subinstances
remove from this browser
change category...
revert to previous version
remove from current change set
revert and forget
fetch documentation
run this
more...' 
	lines:  #(5 7 9 11 14)
	selections: #(classHierarchy browseClass 
		openSingleMessageBrowser makeIsolatedCodePane toggleDiffing browseAllMessages findMethodInChangeSets 
		inspectInstances inspectSubInstances
		removeMessageFromBrowser 
		changeCategory
		revertToPreviousVersion 
		removeFromCurrentChanges revertAndForget 
		fetchDocPane
		runSelectedMethod
		unshiftedYellowButtonActivity
		)]
! !

!Browser methodsFor: 'message functions' stamp: 'ls 6/5/2000 15:27'!
toggleMessagePrivilage
	"toggle whether the method should be compiled with privilage"
	self selectedClassOrMetaClass ifNil: [ ^self ].
	self selectedMessageName ifNil: [ ^self ].
	self selectedClassOrMetaClass toggleSelectorPrivilage: self selectedMessageName.

	"recompile the method"
	self flag: #xxx.  "I don't know how to do this -lex"! !


!ClassDescription methodsFor: 'initialize-release' stamp: 'ls 6/5/2000 15:43'!
obsolete
	"Make the receiver obsolete."
	superclass removeSubclass: self.
	organization _ nil.
	fullyPrivilagedSelectors _ nil.
	super obsolete.! !

!ClassDescription methodsFor: 'method dictionary' stamp: 'ls 6/5/2000 15:38'!
removeSelector: selector 
	| priorMethod | 
	"Remove the message whose selector is given from the method 
	dictionary of the receiver, if it is there. Answer nil otherwise."

	(self methodDict includesKey: selector) ifFalse: [^ nil].
	priorMethod _ self compiledMethodAt: selector.
	Smalltalk changes removeSelector: selector class: self
		priorMethod: priorMethod
		lastMethodInfo: {priorMethod sourcePointer.
						(self whichCategoryIncludesSelector: selector)}.
	self removeSelectorUnlogged: selector.
	self acceptsLoggingOfCompilation ifTrue:
		[Smalltalk logChange: self name , ' removeSelector: #' , selector]! !

!ClassDescription methodsFor: 'method dictionary' stamp: 'ls 6/5/2000 15:39'!
removeSelectorUnlogged: aSymbol 
	"Remove the message whose selector is aSymbol from the method dictionary of the receiver, if it is there. Answer nil otherwise.  Do not log the action either to the current change set or to the changes log"

	(self methodDict includesKey: aSymbol) ifFalse: [^ nil].
	super removeSelector: aSymbol.
	self organization removeElement: aSymbol.
	fullyPrivilagedSelectors ifNotNil: [
		fullyPrivilagedSelectors remove: aSymbol ifAbsent: [] ].! !

!ClassDescription methodsFor: 'fileIn/Out' stamp: 'ls 6/5/2000 15:36'!
printMethodChunk: selector withPreamble: doPreamble on: outStream
		moveSource: moveSource toFile: fileIndex
	"Copy the source code for the method associated with selector onto the fileStream.  If moveSource true, then also set the source code pointer of the method."
	| preamble method oldPos newPos sourceFile endPos |
	"write out the privilage specification for the method"
	outStream nextChunkPut: (self name, ' makeSelector: ', selector storeString, ' privilaged: ', (self selectorIsPrivilaged: selector) storeString).

	doPreamble 
		ifTrue: [preamble _ self name , ' methodsFor: ' ,
					(self organization categoryOfElement: selector) asString printString]
		ifFalse: [preamble _ ''].
	method _ self methodDict at: selector.
	((method fileIndex = 0
		or: [(SourceFiles at: method fileIndex) == nil])
		or: [(oldPos _ method filePosition) = 0])
		ifTrue:
		["The source code is not accessible.  We must decompile..."
		preamble size > 0 ifTrue: [outStream cr; nextPut: $!!; nextChunkPut: preamble; cr].
		outStream nextChunkPut: (self decompilerClass new decompile: selector
											in: self method: method) decompileString]
		ifFalse:
		[sourceFile _ SourceFiles at: method fileIndex.
		sourceFile position: oldPos.
		preamble size > 0 ifTrue:    "Copy the preamble"
			[outStream copyPreamble: preamble from: sourceFile].
		"Copy the method chunk"
		newPos _ outStream position.
		outStream copyMethodChunkFrom: sourceFile.
		sourceFile skipSeparators.      "The following chunk may have ]style["
		sourceFile peek == $] ifTrue: [
			outStream cr; copyMethodChunkFrom: sourceFile].
		moveSource ifTrue:    "Set the new method source pointer"
			[endPos _ outStream position.
			method checkOKToAdd: endPos - newPos at: newPos.
			method setSourcePosition: newPos inFile: fileIndex]].
	preamble size > 0 ifTrue: [outStream nextChunkPut: ' '].
	^ outStream cr! !

!ClassDescription methodsFor: 'security' stamp: 'ls 8/14/2000 16:14'!
definesPrivilagedMethod
        "whether this behavior defines any privilaged methods.  Does *not* check superclasses!!"
        ^fullyPrivilagedSelectors notNil

! !

!ClassDescription methodsFor: 'security' stamp: 'ls 6/5/2000 14:56'!
makeSelector: aSelector privilaged: aBoolean
	"make aSelector either privilaged or unprivilaged, depending on aBoolean"
	aBoolean 
		ifTrue: [ self makeSelectorPrivilaged: aSelector ] 
		ifFalse: [ self makeSelectorUnprivilaged: aSelector ]! !

!ClassDescription methodsFor: 'security' stamp: 'ls 6/5/2000 14:55'!
makeSelectorPrivilaged: aSelector
	"mark the given selector as privilaged"
	fullyPrivilagedSelectors ifNil: [ fullyPrivilagedSelectors := IdentitySet new ].
	fullyPrivilagedSelectors add: aSelector.! !

!ClassDescription methodsFor: 'security' stamp: 'ls 6/8/2000 10:35'!
makeSelectorUnprivilaged: aSelector
        "mark the given selector as unprivilaged"
        fullyPrivilagedSelectors ifNil: [ ^self ].
        fullyPrivilagedSelectors remove: aSelector ifAbsent: [].
        fullyPrivilagedSelectors size = 0 ifTrue: [ fullyPrivilagedSelectors _ nil ].! !

!ClassDescription methodsFor: 'security' stamp: 'ls 6/7/2000 10:43'!
selectorIsPrivilaged: selector
	fullyPrivilagedSelectors ifNil: [ ^false ].
	^fullyPrivilagedSelectors includes: selector.! !

!ClassDescription methodsFor: 'security' stamp: 'ls 6/5/2000 14:56'!
toggleSelectorPrivilage: aSelector
	"toggle whether the selector is privilaged or not"
	self 
		makeSelector: aSelector 
		privilaged: (self selectorIsPrivilaged: aSelector) not! !


!DeepCopier methodsFor: 'like fullCopy' stamp: 'ls 6/9/2000 12:37'!
checkVariables
	"Check that no indexes of instance vars have changed in certain classes.  If you get an error in this method, an implementation of veryDeepCopyWith: needs to be updated.  The idea is to catch a change while it is still in the system of the programmer who made it.  
	DeepCopier new checkVariables	"

	| str str2 meth |
	str _ '|veryDeepCopyWith: or veryDeepInner: is out of date.'.
	Object instSize = 0 ifFalse: [self error: 'Many implementers of veryDeepCopyWith: are out of date'].
	Morph superclass == Object ifFalse: [self error: 'Morph', str].
	(Morph instVarNames copyFrom: 1 to: 6) = #('bounds' 'owner' 'submorphs' 
			'fullBounds' 'color' 'extension') 
		ifFalse: [self error: 'Morph', str].	"added ones are OK"

	"Every class that implements veryDeepInner: must copy all its inst vars.  Danger is that a user will add a new instance variable and forget to copy it.  So check that the last one is mentioned in the copy method."
	(Smalltalk allClassesImplementing: #veryDeepInner:) do: [:aClass | 
		((aClass compiledMethodAt: #veryDeepInner:) writesField: aClass instSize) ifFalse: [
			aClass instSize > 0 ifTrue: [
				self warnIverNotCopiedIn: aClass sel: #veryDeepInner:]]].
	(Smalltalk allClassesImplementing: #veryDeepCopyWith:) do: [:aClass | 
		meth _ aClass compiledMethodAt: #veryDeepCopyWith:.
		(meth size > 20) & (meth literals includes: #veryDeepCopyWith:) not ifTrue: [
			(meth writesField: aClass instSize) ifFalse: [
				self warnIverNotCopiedIn: aClass sel: #veryDeepCopyWith:]]].

	str2 _ 'Player|copyUniClass and DeepCopier|mapUniClasses are out of date'.
	Behavior instVarNames = #('superclass' 'methodDict' 'format' )
		ifFalse: [self error: str2].
	ClassDescription instVarNames = #('instanceVariables' 'organization' 'fullyPrivilagedSelectors')
		ifFalse: [self error: str2].
	Class instVarNames = #('subclasses' 'name' 'classPool' 'sharedPools' 'environment' 'category' )
		ifFalse: [self error: str2].
	Model superclass == Object ifFalse: [self error: str2].
	Player superclass == Model ifFalse: [self error: str2].
	Model class instVarNames = #() ifFalse: [self error: str2].
	Player class instVarNames = #('scripts' 'slotInfo')
		ifFalse: [self error: str2].
! !


!SystemDictionary methodsFor: 'browsing' stamp: 'ls 6/22/2000 12:04'!
browsePrivilagedMessages
	self browseMessageList: self allPrivilagedMessages name: 'Privilaged Messages'! !

!SystemDictionary methodsFor: 'retrieving' stamp: 'ls 6/22/2000 11:13'!
allPrivilagedMessages
	"return a list of messages which have the privilaged flag turned on"
	| messages |
	messages := OrderedCollection new.
	self allBehaviorsDo: [ :class |
		class selectorsDo: [ :sel |
			(class selectorIsPrivilaged: sel) ifTrue: [
				messages add: (class printString, ' ', sel) ] ] ].

	^messages! !


!Text methodsFor: 'emphasis' stamp: 'ls 6/5/2000 14:46'!
makeSelectorBoldIn: aClass
	"For formatting Smalltalk source code, set the emphasis of that portion of 
	the receiver's string that parses as a message selector to be bold."

	| parser selector endOfSelector |
	string size = 0 ifTrue: [^self].
	parser _ aClass parserClass new.
	selector _ parser parseSelector: string.
	endOfSelector _ parser endOfLastToken min: string size.

	self addAttribute: TextEmphasis bold from: 1 to: endOfSelector.


	"color it red if the method is privilaged"
	(aClass selectorIsPrivilaged: selector)
		ifTrue: [ self addAttribute: TextColor red from: 1 to: endOfSelector ]


! !

!Text methodsFor: 'attributes' stamp: 'ls 6/5/2000 14:44'!
unembellished 
	"Return true if the only emphases are the default font, bold, and red"
	| font1 bold red |
	font1 _ TextFontChange defaultFontChange.
	bold _ TextEmphasis bold.
	red _ TextColor red.
	Preferences ignoreStyleIfOnlyBold ifFalse:
		["Ignore font1 only or font1-bold followed by font1-plain"
		^ (runs values = (Array with: (Array with: font1)))
		or: [runs values = (Array with: (Array with: font1 with: bold)
 								with: (Array with: font1))]].

	"If preference is set, then ignore any combo of font1 and bold"
	runs withStartStopAndValueDo:
		[:start :stop :emphArray |
		emphArray do:
			[:emph | (font1 = emph or: [bold = emph or: [ red = emph ]]) ifFalse: [^ false]]].
	^ true! !

