'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 12:00:03 pm'!
"Change Set:		RemoveErrorHandler
Date:			9 June 2000
Author:			Lex Spoon

Removes the redundant errorHandler instvar of Process.  errorHandler is just a special case of handling Error exceptions"!

Link subclass: #Process
	instanceVariableNames: 'suspendedContext priority myList errorHandler island '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!



BlockContext makeSelector: #ifError: privilaged: false!
!BlockContext methodsFor: 'evaluating' stamp: 'ls 6/9/2000 11:40'!
ifError: errorHandlerBlock
	"Evaluate the block represented by the receiver. If an error occurs the given is evaluated with the error message and the receiver as parameters. The error handler block may return a value to be used if the receiver block gets an error. The receiver should not contain an explicit return statement as this would leave an obsolete error handler hanging around."
	"Note: it is more flexible to use #on:do: instead.  #on:do: allows trapping only specific errors, and it allows querying more details on what exactly happened"

	"Examples:
		[1 whatsUpDoc] ifError: [:err :rcvr | ^ 'huh?'].
		[1 / 0] ifError: [:err :rcvr |
			'ZeroDivide' = err
				ifTrue: [^ Float infinity]
				ifFalse: [self error: err]]
"

	^self on: Error do: [:ex |
		^errorHandlerBlock value: ex description value: ex receiver].


! !
BlockContext makeSelector: #value:ifError: privilaged: false!
!BlockContext methodsFor: 'evaluating' stamp: 'ls 6/9/2000 11:41'!
value: arg1 ifError: aBlock
	"Evaluate the block represented by the receiver. If an error occurs aBlock is evaluated
	 with the error message and the receiver as parameters"

	^[ self value: arg1 ] on: Error do: aBlock! !

WorldState makeSelector: #displayWorldSafely: privilaged: false!
!WorldState methodsFor: 'update cycle' stamp: 'ls 6/14/2000 14:15'!
displayWorldSafely: aWorld
	"Update this world's display and keep track of errors during draw methods."

	| errCtx errMorph |

	[aWorld displayWorld] on: Error do: [ :ex |
		"Handle a drawing error"
		errCtx _ Debugger myContext.
		[
			errCtx _ errCtx sender.
			"Search the sender chain to find the morph causing the problem"
			[errCtx notNil and:[(errCtx receiver isKindOf: Morph) not]] 
				whileTrue:[errCtx _ errCtx sender].
			"If we're at the root of the context chain then we have a fatal drawing problem"
			errCtx == nil ifTrue:[^self handleFatalDrawingError: ex messageText].
			errMorph _ errCtx receiver.
			"If the morph causing the problem has already the #drawError flag set,
			then search for the next morph above in the caller chain."
			errMorph hasProperty: #errorOnDraw
		] whileTrue.
		errMorph setProperty: #errorOnDraw toValue: true.
		"Re-raise the error"
		ex pass.
	].! !



Process removeSelector: #errorHandler!
Process removeSelector: #errorHandler:!
Link subclass: #Process
	instanceVariableNames: 'suspendedContext priority myList island '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!
Object removeSelector: #cannotInterpret:!
