'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 11:59:37 am'!
"Change Set:		RestrictedProxy
Date:			26 June 2000
Author:			Lex Spoon

Defines a class where instance variables really are protected from outside access"!

Object subclass: #RestrictedProxy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sandbox-Proxies'!

!RestrictedProxy commentStamp: 'ls 8/14/2000 14:50' prior: 0!
An canonical proxy.  Instances don't actually do anything useful, but they do disable the commands that are normally unsafe for proxies.  Simple proxies may be implemented as a subclass of this class; more complex ones may need to duplicate all of this class's methods.!
Object makeSelector: #weakClass privilaged: true!
!Object methodsFor: 'class membership' stamp: 'ls 8/4/2000 15:46'!
weakClass
	"Answer an object responding to the object's class methods.  Depending on the Island that is in force, the return value may be the actual implementation class.  If the island is restrictive, then the returned object will not allow instantiation"
	^!!Smalltalk currentIsland weakClassProxyFor:  (!!ObjectInspector classOf: self).! !
Object makeSelector: #safeColorFor: privilaged: true!
!Object methodsFor: 'safe conversion' stamp: 'ls 8/4/2000 16:31'!
safeColorFor: inputObject
	"returns a completely new object which is definately a color"
	^!!Color
		r: (self safeIntegerFor: inputObject red )
		g: (self safeIntegerFor: inputObject green)
		b: (self safeIntegerFor: inputObject blue)
		alpha: (self safeIntegerFor: inputObject alpha)
! !
Object makeSelector: #safeIntegerFor: privilaged: true!
!Object methodsFor: 'safe conversion' stamp: 'ls 8/11/2000 10:04'!
safeIntegerFor: inputObject
	"returns a completely new object which is definately an integer"
	| inputInteger |
	inputInteger := inputObject asInteger.
	(!!ObjectInspector classOf: inputInteger) == !!SmallInteger ifFalse: [
		self error: 'not a small integer' ].
	^inputInteger
! !
Object makeSelector: #safePointFor: privilaged: true!
!Object methodsFor: 'safe conversion' stamp: 'ls 8/4/2000 16:31'!
safePointFor: inputObject
	"return a point based on the input object which definately contains integer coordinates"
	^!!Point x: (self safeIntegerFor: inputObject x) y: (self safeIntegerFor: inputObject y)! !
Object makeSelector: #safeRectangleFor: privilaged: true!
!Object methodsFor: 'safe conversion' stamp: 'ls 8/4/2000 16:31'!
safeRectangleFor: inputRectangle
	"return a new object which is definately a rectangle"
	^!!Rectangle origin: (self safePointFor: inputRectangle origin) extent: (self safePointFor: inputRectangle extent)! !
Object makeSelector: #safeStringFor:maxLength: privilaged: true!
!Object methodsFor: 'safe conversion' stamp: 'ls 8/4/2000 16:32'!
safeStringFor: inputObject  maxLength: maxLength
        | inputString |
        "returns a completely new object which is definately a string"
        inputString := inputObject asString.
        (!!ObjectInspector classOf: inputString) == !!String ifFalse: [
                self error: 'not a string' ].
        inputString size > maxLength ifTrue: [
                self error: 'string is too long' ].
        ^inputString copy
! !
Object makeSelector: #notAllowed privilaged: false!
!Object methodsFor: 'as yet unclassified' stamp: 'ls 6/28/2000 15:34'!
notAllowed
	self error: 'operation not permitted'! !

RestrictedProxy makeSelector: #class privilaged: false!
!RestrictedProxy methodsFor: 'proxy restrictions' stamp: 'ls 6/28/2000 12:32'!
class
	"Answer an object responding to the object's class methods.  Depending on the Island that is in force, the return value may be the actual implementation class"
	^self weakClass! !
RestrictedProxy makeSelector: #clone privilaged: false!
!RestrictedProxy methodsFor: 'proxy restrictions' stamp: 'ls 8/14/2000 14:35'!
clone
	"a proxy can't be copied"
	^self! !
RestrictedProxy makeSelector: #instVarAt: privilaged: false!
!RestrictedProxy methodsFor: 'proxy restrictions' stamp: 'ls 8/14/2000 14:35'!
instVarAt: index
	^self notAllowed! !
RestrictedProxy makeSelector: #instVarAt:put: privilaged: false!
!RestrictedProxy methodsFor: 'proxy restrictions' stamp: 'ls 8/14/2000 14:35'!
instVarAt: index put: value
	^self notAllowed! !
RestrictedProxy makeSelector: #shallowCopy privilaged: false!
!RestrictedProxy methodsFor: 'proxy restrictions' stamp: 'ls 8/14/2000 14:35'!
shallowCopy
	"a proxy can't be copied"
	^self! !

RestrictedProxy removeSelector: #copy!
RestrictedProxy removeSelector: #doesNotUnderstand:!
RestrictedProxy removeSelector: #error:!
RestrictedProxy removeSelector: #flag:!
RestrictedProxy removeSelector: #isBehavior!
RestrictedProxy removeSelector: #perform:!
RestrictedProxy removeSelector: #perform:with:!
RestrictedProxy removeSelector: #perform:with:with:with:!
RestrictedProxy removeSelector: #perform:withArguments:!
RestrictedProxy removeSelector: #quitProcess!
RestrictedProxy removeSelector: #yourself!

!RestrictedProxy reorganize!
('proxy restrictions' class clone instVarAt: instVarAt:put: shallowCopy)
!

