'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 11:59:47 am'!
"Change Set:		addingPrivilage
Date:			14 June 2000
Author:			Lex Spoon

Sets the privilaged flag on a myriad of messages in the system, updates several accesses to #fork and #newProcess, and changes several references to thisContext to use 'Debugger myContext' instead "!

ObjectTracer makeSelector: #doesNotUnderstand: privilaged: false!
!ObjectTracer methodsFor: 'very few messages' stamp: 'ls 6/14/2000 16:15'!
doesNotUnderstand: aMessage 
	"All external messages (those not caused by the re-send) get trapped here"
	"Present a dubugger before proceeding to re-send the message"

	Debugger openContext: Debugger myContext
				label: 'About to perform: ', aMessage selector
				contents: Debugger myContext shortStack.
	^ aMessage sentTo: tracedObject.
! !

ProtoObject makeSelector: #cannotInterpret: privilaged: false!
!ProtoObject methodsFor: 'system primitives' stamp: 'ls 7/10/2000 15:58'!
cannotInterpret: aMessage 
	 "Handle the fact that there was an attempt to send the given message to the receiver but a null methodDictionary was encountered while looking up the message selector.  Hopefully this is the result of encountering a stub for a swapped out class which induces this exception on purpose."

"If this is the result of encountering a swap-out stub, then simulating the lookup in Smalltalk should suffice to install the class properly, and the message may be resent."

	| |
	(self class lookupSelector: aMessage selector) == nil ifFalse:
		["Simulated lookup succeeded -- resend the message."
		^ aMessage sentTo: self].

	"Could not recover by simulated lookup -- it's an error"
	self error: 'MethodDictionary fault'.

	^ aMessage sentTo: self! !

ImageSegmentRootStub makeSelector: #doesNotUnderstand: privilaged: false!
!ImageSegmentRootStub methodsFor: 'fetch from disk' stamp: 'ls 6/19/2000 16:55'!
doesNotUnderstand: aMessage 
	 "Any normal message sent to this object is really intended for another object that is in a non-resident imageSegment.  Reinstall the segment and resend the message."

	imageSegment install.
	LoggingFaults ifTrue:		"Save the stack printout to show who caused the fault"
		[FaultLogs at: Time millisecondClockValue printString
			put: (String streamContents:
				[:strm | 
				strm print: self class; space; print: aMessage selector; cr.
				(Debugger myContext sender stackOfSize: 30)
					do: [:item | strm print: item; cr]])].

	"NOTE:  The following should really be (aMessage sentTo: self)
		in order to recover properly from a fault in a super-send,
		however, the lookupClass might be bogus in this case, and it's
		almost unthinkable that the first fault would be a super send."
	^ self perform: aMessage selector withArguments: aMessage arguments! !

Object makeSelector: #notify: privilaged: false!
!Object methodsFor: 'error handling' stamp: 'ls 6/27/2000 10:27'!
notify: aString 
	"Create and schedule a Notifier with the argument as the message in 
	order to request confirmation before a process can proceed."

	Debugger
		openContext: Debugger myContext
		label: 'Notifier'
		contents: aString

	"nil notify: 'confirmation message'"! !
Object makeSelector: #tryToDefineVariableAccess: privilaged: false!
!Object methodsFor: 'error handling' stamp: 'ls 8/4/2000 16:20'!
tryToDefineVariableAccess: aMessage
	"See if the message just wants to get at an instance variable of this class.  Ask the user if its OK.  If so, define the message to read or write that instance or class variable and retry."
	| ask newMessage sel |
	aMessage arguments size > 1 ifTrue: [^ false].
	sel _ aMessage selector asString.	"works for 0 args"
	aMessage arguments size = 1 ifTrue: [
		sel last = $: ifFalse: [^ false].
		sel _ sel copyWithout: $:].
	(self class instVarNames includes: sel) ifFalse: [
		(self class classVarNames includes: sel asSymbol) ifFalse: [
			^ false]].
	ask _ self confirm: 'A ', Debugger myContext sender sender receiver 
		class printString, ' wants to ', 
		(aMessage arguments size = 1 ifTrue: ['write into'] ifFalse: ['read from']), '
', sel ,' in class ', self class printString, '.
Define a this access message?'.
	ask ifTrue: [
		aMessage arguments size = 1 
			ifTrue: [newMessage _ aMessage selector, ' anObject
	', sel, ' _ anObject']
			ifFalse: [newMessage _ aMessage selector, '
	^', aMessage selector].
		self class compile: newMessage classified: 'accessing' notifying: nil].
	^ ask! !
Object makeSelector: #primitiveError: privilaged: false!
!Object methodsFor: 'private' stamp: 'ls 6/19/2000 17:12'!
primitiveError: aString 
	"This method is called when the error handling results in a recursion in 
	calling on error: or halt or halt:."

	| context |
	(String
		streamContents: 
			[:s |
			s nextPutAll: '**System error handling failed**'.
			s cr; nextPutAll: aString.
			context _ Debugger myContext sender sender.
			20 timesRepeat: [context == nil ifFalse: [s cr; print: (context _ context sender)]].
			s cr; nextPutAll: '**Type CR to enter an emergency evaluator.**'.
			s cr; nextPutAll: '**Type any other character to restart.**'])
		displayAt: 0 @ 0.
	[Sensor keyboardPressed] whileFalse.
	Sensor keyboard == Character cr ifTrue: [Transcripter emergencyEvaluator].
	Smalltalk isMorphic ifTrue: [
		^Display reinstallMorphicWorldAfterError "To init hand events and redisplay world"
	].
	ScheduledControllers searchForActiveController! !
Object makeSelector: #isThisEverCalled privilaged: false!
!Object methodsFor: 'flagging' stamp: 'ls 6/26/2000 16:50'!
isThisEverCalled
	^ self isThisEverCalled: Debugger myContext sender printString! !
Object makeSelector: #logEntry privilaged: false!
!Object methodsFor: 'flagging' stamp: 'ls 6/19/2000 17:12'!
logEntry

	Transcript show: 'Entered ', Debugger myContext sender printString; cr.
! !
Object makeSelector: #logExecution privilaged: false!
!Object methodsFor: 'flagging' stamp: 'ls 6/19/2000 17:12'!
logExecution

	Transcript show: 'Executing ', Debugger myContext sender printString; cr.
! !
Object makeSelector: #logExit privilaged: false!
!Object methodsFor: 'flagging' stamp: 'ls 6/19/2000 17:12'!
logExit

	Transcript show:  'Exited ', Debugger myContext sender printString; cr.
! !
Object makeSelector: #primitive:parameters:receiver: privilaged: false!
!Object methodsFor: 'translation support' stamp: 'ls 6/19/2000 17:12'!
primitive: primID parameters: parmSpecs receiver: rcvrSpec

	| tMethod |
	tMethod _ TestTMethod new 
		fromContext: Debugger myContext sender 
		primitive: primID 
		parameters: parmSpecs 
		receiver: rcvrSpec.
	^tMethod simulatePrologInContext: Debugger myContext sender! !

ApplescriptError class makeSelector: #open: privilaged: false!
!ApplescriptError class methodsFor: 'as yet unclassified' stamp: 'ls 6/14/2000 16:50'!
open: aSyntaxError
	"Answer a standard system view whose model is an instance of me."

	| topView |
	self simulationGuard.
	Smalltalk isMorphic
		ifTrue:
			[self buildMorphicViewOn: aSyntaxError.
			CurrentProjectRefactoring newProcessIfUI: Processor activeProcess.
			^ Processor activeProcess suspend].
	topView _ self buildMVCViewOn: aSyntaxError.
	topView controller openNoTerminateDisplayAt: Display extent // 2.
	Cursor normal show.
	Processor activeProcess suspend! !

Collection makeSelector: #toBraceStack: privilaged: false!
!Collection methodsFor: 'private' stamp: 'ls 6/19/2000 16:40'!
toBraceStack: itsSize 
	"Push receiver's elements onto the stack of thisContext sender.  Error if receiver does
	 not have itsSize elements or if receiver is unordered.
	 Do not call directly: this is called by {a. b} _ ... constructs."

	self size ~= itsSize ifTrue:
		[self error: 'Trying to store ', self size printString,
					' values into ', itsSize printString, ' variables.'].
	Debugger myContext  sender push: itsSize fromIndexable: self! !

CompiledMethod makeSelector: #copy privilaged: true!
!CompiledMethod methodsFor: 'initialize-release' stamp: 'ls 8/14/2000 17:57'!
copy
	| copy end start |
	start _ self initialPC.
	end _ self endPC.
	copy _ !!CompiledMethod newMethod: end - start + 1
				header: self header.
	1 to: self numLiterals do: [:i | copy literalAt: i put: (self literalAt: i)].
	start to: end do: [:i | copy at: i put: (self at: i)].

	^ copy! !

ComplexProgressIndicator makeSelector: #backgroundWorldDisplay privilaged: false!
!ComplexProgressIndicator methodsFor: 'as yet unclassified' stamp: 'ls 7/10/2000 17:41'!
backgroundWorldDisplay

	| f |

	self flag: #bob.		"really need a better way to do this"

			"World displayWorldSafely."

	"ugliness to try to track down a possible error"


	[World displayWorld] ifError: [ :a :b |
		stageCompleted _ 999.
		f _ FileDirectory default fileNamed: 'bob.errors'.
		f nextPutAll: a printString,'  ',b printString; cr; cr.
		f nextPutAll: 'worlds equal ',(formerWorld == World) printString; cr; cr.
		f nextPutAll: Debugger myContext longStack; cr; cr.
		f nextPutAll: formerProcess suspendedContext longStack; cr; cr.
		f close. 1 beep.
	].
! !

ContextPart makeSelector: #restrictedBlockCopy: privilaged: true!
!ContextPart methodsFor: 'controlling' stamp: 'ls 8/14/2000 12:01'!
restrictedBlockCopy: numArgs 
	"Primitive. Distinguish a block of code from its enclosing method by 
	creating a new BlockContext for that block. The compiler inserts into all 
	methods that contain blocks the bytecodes to send the message 
	blockCopy:. Do not use blockCopy: in code that you write!! Only the 
	compiler can decide to send the message blockCopy:. Fail if numArgs is 
	not a SmallInteger. Optional. No Lookup. See Object documentation 
	whatIsAPrimitive."
	| block |
	block := (!!BlockContext newForMethod: self home method)
		home: self home
		startpc: pc + 2
		nargs: numArgs.
	^!!RestrictedBlock on: block
! !

ContextPart class makeSelector: #tallyInstructions: privilaged: false!
!ContextPart class methodsFor: 'examples' stamp: 'ls 6/19/2000 16:06'!
tallyInstructions: aBlock
	"This method uses the simulator to count the number of occurrences of
	each of the Smalltalk instructions executed during evaluation of aBlock.
	Results appear in order of the byteCode set."
	| tallies |
	tallies _ Bag new.
	Debugger myContext sender
		runSimulated: aBlock
		contextAtEachStep:
			[:current | tallies add: current nextByte].
	^tallies sortedElements

	"ContextPart tallyInstructions: [3.14159 printString]"! !
ContextPart class makeSelector: #tallyMethods: privilaged: false!
!ContextPart class methodsFor: 'examples' stamp: 'ls 6/19/2000 16:07'!
tallyMethods: aBlock
	"This method uses the simulator to count the number of calls on each method
	invoked in evaluating aBlock. Results are given in order of decreasing counts."
	| prev tallies |
	tallies _ Bag new.
	prev _ aBlock.
	Debugger myContext sender
		runSimulated: aBlock
		contextAtEachStep:
			[:current |
			current == prev ifFalse: "call or return"
				[prev sender == nil ifFalse: "call only"
					[tallies add: current printString].
				prev _ current]].
	^tallies sortedCounts

	"ContextPart tallyMethods: [3.14159 printString]"! !
ContextPart class makeSelector: #trace:on: privilaged: false!
!ContextPart class methodsFor: 'examples' stamp: 'ls 6/19/2000 16:06'!
trace: aBlock on: aStream		"ContextPart trace: [3 factorial]"
	"This method uses the simulator to print calls to a file."
	| prev |
	prev _ aBlock.
	^ Debugger myContext sender
		runSimulated: aBlock
		contextAtEachStep:
			[:current |
			Sensor anyButtonPressed ifTrue: [^ nil].
			current == prev
				ifFalse:
					[prev sender ifNil:
						[aStream space; nextPut: $^.
						self carefullyPrint: current top on: aStream].
					aStream cr.
					(current depthBelow: aBlock) timesRepeat: [aStream space].
					self carefullyPrint: current receiver on: aStream.
					aStream space; nextPutAll: current selector; flush.
					prev _ current]]! !
ContextPart class makeSelector: #runSimulated: privilaged: false!
!ContextPart class methodsFor: 'simulation' stamp: 'ls 6/19/2000 16:06'!
runSimulated: aBlock
	"Simulate the execution of the argument, current. Answer the result it 
	returns."

	^Debugger myContext sender
		runSimulated: aBlock
		contextAtEachStep: [:ignored]

	"ContextPart runSimulated: [Pen new defaultNib: 5; go: 100]"! !

Form makeSelector: #primPrintHScale:vScale:landscape: privilaged: false!
!Form methodsFor: 'other' stamp: 'ls 6/22/2000 11:18'!
primPrintHScale: hScale vScale: vScale landscape: aBoolean
	"On platforms that support it, this primitive prints the receiver, assumed to be a Form, to the default printer."
	"(Form extent: 10@10) primPrintHScale: 1.0 vScale: 1.0 landscape: true"

	self flag: #sandbox.
	self error: 'needs to be relocated for the sandbox changes--accessing a form isn''t enough to get privilage to print it out'! !

DisplayScreen makeSelector: #findAnyDisplayDepth privilaged: false!
!DisplayScreen methodsFor: 'private' stamp: 'ls 6/19/2000 16:46'!
findAnyDisplayDepth
	"Return any display depth that is supported on this system."
	^self findAnyDisplayDepthIfNone:[
		"Ugh .... now this is a biggie - a system that does not support
		any of the Squeak display depths at all."
		Smalltalk
			logError:'Fatal error: This system has no support for any display depth at all.'
			inContext: Debugger myContext
			to: 'SqueakDebug.log'.
		Smalltalk quitPrimitive. "There is no way to continue from here"
	].! !

HTMLformatter class makeSelector: #oldEvalEmbedded:with: privilaged: false!
!HTMLformatter class methodsFor: 'translating' stamp: 'ls 6/14/2000 15:51'!
oldEvalEmbedded: stringOrStream with: request
	| sourceStream targetStream
evalStream currentStream evalValue peekValue ch |
	(stringOrStream
isKindOf: Stream)
		ifTrue: [sourceStream := stringOrStream]

		ifFalse: [sourceStream := ReadStream on: stringOrStream].

	targetStream := WriteStream on: String new.
	currentStream :=
targetStream. 
	[sourceStream atEnd] whileFalse: 
		[ch :=
sourceStream next.
		ch = $< ifTrue:
			[
peekValue := sourceStream peek. (peekValue = $?) ifTrue:

		[evalStream := WriteStream on: String new.

		currentStream := evalStream.

	sourceStream next. "Eat the ?"
				ch :=
sourceStream next.]].
		((currentStream = evalStream) and: [ch =
$?]) ifTrue:
			[ peekValue := sourceStream peek.
(peekValue = $>) ifTrue:
				[sourceStream next.
"Eat the >"
				currentStream := targetStream.

			evalValue := (Compiler new evaluate: (evalStream
contents) 
					in: nil to: self
notifying: nil ifFail: [^nil]).
				(evalValue
isKindOf: String)
				ifFalse: [evalValue :=
evalValue printString].
				currentStream
nextPutAll: evalValue.]]
			ifFalse: [currentStream
nextPut: ch].].
	^targetStream contents

! !

InterpreterPlugin class makeSelector: #doPrimitive: privilaged: false!
!InterpreterPlugin class methodsFor: 'instance creation' stamp: 'ls 6/19/2000 17:01'!
doPrimitive: primitiveName 
	| proxy plugin |
	proxy _ InterpreterProxy new.
	proxy loadStackFrom: Debugger myContext sender.
	plugin _ self simulatorClass new.
	plugin setInterpreter: proxy.
	plugin perform: primitiveName asSymbol.
	^ proxy stackValue: 0! !

BitBltSimulation class makeSelector: #copyBitsFrom: privilaged: false!
!BitBltSimulation class methodsFor: 'system simulation' stamp: 'ls 6/19/2000 16:35'!
copyBitsFrom: aBitBlt
	"Simulate the copyBits primitive"
	| proxy bb |
	proxy _ InterpreterProxy new.
	proxy loadStackFrom: Debugger myContext sender.
	bb _ self simulatorClass new.
	bb setInterpreter: proxy.
	proxy success: (bb loadBitBltFrom: aBitBlt).
	bb copyBits.
	proxy failed ifFalse:[
		proxy showDisplayBits: aBitBlt destForm 
				Left: bb affectedLeft Top: bb affectedTop 
				Right: bb affectedRight Bottom: bb affectedBottom].
	^proxy stackValue: 0! !
BitBltSimulation class makeSelector: #warpBitsFrom: privilaged: false!
!BitBltSimulation class methodsFor: 'system simulation' stamp: 'ls 6/19/2000 16:35'!
warpBitsFrom: aBitBlt
	"Simulate the warpBits primitive"
	| proxy bb |
	proxy _ InterpreterProxy new.
	proxy loadStackFrom: Debugger myContext  sender.
	bb _ self simulatorClass new.
	bb setInterpreter: proxy.
	proxy success: (bb loadWarpBltFrom: aBitBlt).
	bb warpBits.
	proxy failed ifFalse:[
		proxy showDisplayBits: aBitBlt destForm 
				Left: bb affectedLeft Top: bb affectedTop 
				Right: bb affectedRight Bottom: bb affectedBottom].
	^proxy stackValue: 0! !

FXBltSimulation class makeSelector: #copyBitsFrom: privilaged: false!
!FXBltSimulation class methodsFor: 'system simulation' stamp: 'ls 6/19/2000 16:49'!
copyBitsFrom: aBitBlt
	"Simulate the copyBits primitive"
	| proxy bb |
	proxy _ InterpreterProxy new.
	proxy loadStackFrom: Debugger myContext sender.
	bb _ self simulatorClass new.
	bb setInterpreter: proxy.
	proxy success: (bb loadBitBltFrom: aBitBlt).
	bb copyBits.
	proxy failed ifFalse:[
		proxy showDisplayBits: aBitBlt destForm 
				Left: bb affectedLeft Top: bb affectedTop 
				Right: bb affectedRight Bottom: bb affectedBottom].
	^proxy stackValue: 0! !
FXBltSimulation class makeSelector: #fxCopyBitsFrom: privilaged: false!
!FXBltSimulation class methodsFor: 'system simulation' stamp: 'ls 6/19/2000 16:49'!
fxCopyBitsFrom: aBitBlt
	"Simulate the copyBits primitive"
	| proxy bb |
	proxy _ InterpreterProxy new.
	proxy loadStackFrom: Debugger myContext sender.
	bb _ self simulatorClass new.
	bb setInterpreter: proxy.
	proxy success: (bb loadBitBltFrom: aBitBlt).
	bb copyBits.
	proxy failed ifFalse:[
		proxy showDisplayBits: aBitBlt destForm 
				Left: bb affectedLeft Top: bb affectedTop 
				Right: bb affectedRight Bottom: bb affectedBottom].
	^proxy stackValue: 0! !
FXBltSimulation class makeSelector: #warpBitsFrom: privilaged: false!
!FXBltSimulation class methodsFor: 'system simulation' stamp: 'ls 6/19/2000 16:49'!
warpBitsFrom: aBitBlt
	"Simulate the warpBits primitive"
	| proxy bb |
	proxy _ InterpreterProxy new.
	proxy loadStackFrom: Debugger myContext sender.
	bb _ self simulatorClass new.
	bb setInterpreter: proxy.
	proxy success: (bb loadWarpBltFrom: aBitBlt).
	bb copyBits.
	proxy failed ifFalse:[
		proxy showDisplayBits: aBitBlt destForm 
				Left: bb affectedLeft Top: bb affectedTop 
				Right: bb affectedRight Bottom: bb affectedBottom].
	^proxy stackValue: 0! !

MessageTally makeSelector: #spyEvery:on: privilaged: false!
!MessageTally methodsFor: 'initialize-release' stamp: 'ls 7/11/2000 15:31'!
spyEvery: millisecs on: inputBlock 
	"Create a spy and spy on the given block at the specified rate."

	| myDelay value startTime time0 aBlock |
	aBlock := ObjectInspector realBlockFor: inputBlock.
	(aBlock isMemberOf: BlockContext)
		ifFalse: [self error: 'spy needs a block here'].
	self class: aBlock receiver class method: aBlock method.
		"set up the probe"
	ObservedProcess _ Processor activeProcess.
	myDelay := Delay forMilliseconds: millisecs.
	time0 := Time millisecondClockValue.
	Timer :=
		Process forBlock: [[true] whileTrue: 
			[startTime := Time millisecondClockValue.
			myDelay wait.
			self tally: ObservedProcess suspendedContext
				"tally can be > 1 if ran a long primitive"
				by: (Time millisecondClockValue - startTime) // millisecs].
		nil].
	Timer priority: Processor userInterruptPriority.
		"activate the probe and evaluate the block"
	Timer resume.
	value := aBlock value.
		"cancel the probe and return the value"
	Timer terminate.
	time := Time millisecondClockValue - time0.
	^value! !

MessageTally class makeSelector: #tallySendsTo:inBlock:showTree: privilaged: false!
!MessageTally class methodsFor: 'spying' stamp: 'ls 6/19/2000 17:10'!
tallySendsTo: receiver inBlock: aBlock showTree: treeOption
	"MessageTally tallySends: [3.14159 printString]"
	"This method uses the simulator to count the number of calls on each method
	invoked in evaluating aBlock. If receiver is not nil, then only sends
	to that receiver are tallied.
	Results are presented as leaves, sorted by frequency,
	preceded, optionally, by the whole tree."
	| prev tallies startTime totalTime |
	startTime _ Time millisecondClockValue.
	tallies _ MessageTally new class: aBlock receiver class
							method: aBlock method.
	prev _ aBlock.
	  Debugger myContext sender
		runSimulated: aBlock
		contextAtEachStep:
			[:current |
			current == prev ifFalse: 
				["call or return"
				prev sender == nil ifFalse: 
					["call only"
					(receiver == nil or: [current receiver == receiver])
						ifTrue: [tallies tally: current by: 1]].
				prev _ current]].

	totalTime _ Time millisecondClockValue - startTime // 1000.0 roundTo: 0.01.
	(StringHolder new contents:
		(String streamContents:
			[:s |
			s nextPutAll: 'This simulation took ' , totalTime printString
							, ' seconds.'; cr.
			treeOption
				ifTrue: [tallies fullPrintOn: s tallyExact: true orThreshold: 0]
				ifFalse: [tallies leavesPrintOn: s tallyExact: true orThreshold: 0].
			tallies close]))
		openLabel: 'Spy Results'! !

MethodContext makeSelector: #cannotReturn: privilaged: false!
!MethodContext methodsFor: 'private-exceptions' stamp: 'ls 6/19/2000 17:10'!
cannotReturn: result

	Debugger
		openContext: Debugger myContext
		label: 'computation has been terminated'
		contents: Debugger myContext printString! !

ObjectOut makeSelector: #xxxFixup privilaged: false!
!ObjectOut methodsFor: 'fetch from disk' stamp: 'ls 6/19/2000 17:14'!
xxxFixup
	"There is already an object in memory for my url.  All pointers to me need to be pointers to him.  Can't use become, because other pointers to him must stay valid."

	| real temp list |
	real _ page contentsMorph.
	real == self ifTrue: [page error: 'should be converted by now'].
	temp _ self.
	list _ (Smalltalk pointersTo: temp) asOrderedCollection.
	list add: Debugger myContext.  list add: Debugger myContext sender.
	list do: [:holder |
		1 to: holder class instSize do:
			[:i | (holder instVarAt: i) == temp ifTrue: [holder instVarAt: i put: real]].
		1 to: holder basicSize do:
			[:i | (holder basicAt: i) == temp ifTrue: [holder basicAt: i put: real]].
		].
	^ real! !

Parser makeSelector: #correctVariable:interval: privilaged: false!
!Parser methodsFor: 'error correction' stamp: 'ls 6/19/2000 16:04'!
correctVariable: proposedVariable interval: spot
	"Correct the proposedVariable to a known variable, or declare it as a new
	variable if such action is requested.  We support declaring lowercase
	variables as temps, and uppercase variables as Globals or ClassVars,
	depending on whether the context is nil (class=UndefinedObject).
	Spot is the interval within the test stream of the variable."

	| alternatives aStream choice userSelection temp binding globalToo |
	"If we can't ask the user for correction, make it undeclared"
proposedVariable = 'thisContext' ifTrue: [ self notify: 'not privilaged' ].
	self interactive ifFalse: [^ encoder undeclared: proposedVariable].

	temp _ proposedVariable first isLowercase.
	"First check to see if the requestor knows anything about the variable"
	(temp and: [(binding _ requestor bindingOf: proposedVariable) notNil])
		ifTrue: [^ encoder global: binding name: proposedVariable].
	userSelection _ requestor selectionInterval.
	requestor selectFrom: spot first to: spot last.
	requestor select.

	alternatives _ encoder possibleVariablesFor: proposedVariable.

	aStream _ WriteStream on: (String new: 200).
	globalToo _ 0.
	aStream nextPutAll: 'declare ' ,
		(temp ifTrue: ['temp']
			ifFalse: [encoder classEncoding == UndefinedObject
					ifTrue: ['Global']
					ifFalse: [globalToo _ 1.  'Class Variable']]); cr.
	globalToo = 1 ifTrue: [aStream nextPutAll: 'declare Global'; cr].
	alternatives do:
		[:sel | aStream nextPutAll: sel; cr].
	aStream nextPutAll: 'cancel'.

	choice _ (PopUpMenu
				labels: aStream contents
				lines: (Array with: (globalToo + 1) with: (globalToo + alternatives size + 1)))
		startUpWithCaption:
(('Unknown variable: ', proposedVariable, '
please correct, or cancel:') asText makeBoldFrom: 19 to: 19 + proposedVariable size).
	(choice = 0) | (choice > (globalToo + alternatives size + 1))
		ifTrue: [^ self fail].

	requestor deselect.
	requestor selectInvisiblyFrom: userSelection first to: userSelection last.
	choice =1 ifTrue:
			[temp ifTrue: [^ self declareTempAndPaste: proposedVariable]
				ifFalse: [encoder classEncoding == UndefinedObject
					ifTrue: [^ self declareGlobal: proposedVariable]
					ifFalse: [^ self declareClassVar: proposedVariable]]].
	(choice = 2) & (globalToo = 1) ifTrue: [^ self declareGlobal: proposedVariable].
	"Spelling correction"
	self substituteWord: (alternatives at: choice-1-globalToo)
			wordInterval: spot
			offset: 0.
	^ encoder encodeVariable: (alternatives at: choice-1-globalToo)! !

Process makeSelector: #terminate privilaged: true!
!Process methodsFor: 'changing process state' stamp: 'ls 8/12/2000 13:28'!
terminate 
	"Stop the process that the receiver represents forever."

	| context |
	!!Processor activeProcess == self
		ifTrue: 
			[thisContext unwindTo: nil.
			thisContext sender == nil ifFalse:
				[thisContext sender release].
			thisContext removeSelf suspend]
		ifFalse: 
			[myList == nil
				ifFalse: 
					[myList remove: self ifAbsent: [].
					myList _ nil].
			context _ suspendedContext.
			suspendedContext _ nil.
			context == nil ifFalse: [context unwindTo: nil].
			(context ~~ nil and: [context sender ~~ nil])
				ifTrue: [context sender release]]! !

PseudoContext class makeSelector: #contextCacheDepth privilaged: false!
!PseudoContext class methodsFor: 'private' stamp: 'ls 6/14/2000 16:23'!
contextCacheDepth
	"Answer the number of entries in the context cache.  This requires a little insider
	knowledge.  Not intended for casual use, which is why it's private protocol."

	^self contextCacheDepth: Debugger myContext yourself! !

SyntaxError makeSelector: #notify:at:in: privilaged: false!
!SyntaxError methodsFor: 'other' stamp: 'ls 7/11/2000 09:27'!
notify: error at: location in: source
	"Open a syntax error view, inserting the given error message into the given source at the given location. This message is sent to the 'requestor' when the parser or compiler finds a syntax error."

	| aClass aString |
	aClass _ Debugger myContext sender receiver encoder classEncoding.
	aString _
		source contents
			copyReplaceFrom: location
			to: location - 1
			with: error.
	self setClass: aClass
		code: aString
		debugger: (Debugger context: Debugger myContext)
		doitFlag: false.
	self class open: self.
! !

SyntaxError class makeSelector: #errorInClass:withCode:doitFlag: privilaged: false!
!SyntaxError class methodsFor: 'instance creation' stamp: 'ls 6/14/2000 16:37'!
errorInClass: aClass withCode: codeString doitFlag: doit
	"Open a view whose model is a syntax error. The error occurred when trying to add the given method code to the given class."

	self open:
		(self new setClass: aClass
			code: codeString
			debugger: (Debugger context: Debugger myContext)
			doitFlag: doit).
! !
SyntaxError class makeSelector: #open: privilaged: false!
!SyntaxError class methodsFor: 'instance creation' stamp: 'ls 6/14/2000 16:42'!
open: aSyntaxError
	"Answer a standard system view whose model is an instance of me."
	| topView |
	self simulationGuard.
	Smalltalk isMorphic
		ifTrue:
			[self buildMorphicViewOn: aSyntaxError.
			CurrentProjectRefactoring newProcessIfUI: Processor activeProcess.
			^ Processor activeProcess suspend].
	topView _ self buildMVCViewOn: aSyntaxError.
	topView controller openNoTerminateDisplayAt: Display extent // 2.
	Cursor normal show.
	Processor activeProcess suspend.
! !

SystemDictionary makeSelector: #pointersTo:except: privilaged: false!
!SystemDictionary methodsFor: 'retrieving' stamp: 'ls 6/27/2000 10:46'!
pointersTo: anObject except: objectsToExclude
	"Find all occurrences in the system of pointers to the argument anObject. Remove objects in the exclusion list from the results."

	| results anObj |
	Smalltalk garbageCollect.
	"big collection shouldn't grow, so it's contents array is always the same"
	results _ OrderedCollection new: 1000.

	"allObjectsDo: is expanded inline to keep spurious
	 method and block contexts out of the results"
	anObj _ self someObject.
	[0 == anObj] whileFalse: [
		anObj isInMemory ifTrue: [
			(anObj pointsTo: anObject) ifTrue: [
				"exclude the results collector and contexts in call chain"
				((anObj ~~ results collector) and:
				 [(anObj ~~ objectsToExclude) and:
				 [(anObj ~~ Debugger myContext) and:
				 [(anObj ~~ Debugger myContext sender) and:
				 [anObj ~~ Debugger myContext sender sender]]]])
					 ifTrue: [ results add: anObj ].
			]].
		anObj _ self nextObjectAfter: anObj.
	].
	objectsToExclude do: [ :obj | results removeAllSuchThat: [ :el | el == obj]].

	^ results asArray
! !
SystemDictionary makeSelector: #lowSpaceThreshold privilaged: false!
!SystemDictionary methodsFor: 'memory space' stamp: 'ls 6/19/2000 17:27'!
lowSpaceThreshold
	"Return the low space threshold. When the amount of free memory (after garbage collection) falls below this limit, the system is in serious danger of completely exhausting memory and crashing. This limit should be made high enough to allow the user open a debugger to diagnose a problem or to save the image."

	Debugger myContext isPseudoContext
		ifTrue: [^ 300000  "Allow for translated methods"]
		ifFalse: [^ 100000  "Enough for interpreter"]! !

SystemTracer makeSelector: #doit privilaged: false!
!SystemTracer methodsFor: 'initialization' stamp: 'ls 6/19/2000 17:29'!
doit  "SystemTracer writeClone"
	"(StandardFileStream allInstances select:
		[:f | f name endsWith: 'clone.image']) do: [:f | f close]."
	| time1 time2 ctxt n |
	ctxt _ Debugger myContext sender.
	ctxt push: nil.
	self init: ctxt.
	cleaningUp _ false.	"true means rewriting special objects in writeSpecial2"
	Transcript show: 'Tracing . . . '.
	time1 _ Time millisecondClockValue.
	file _ FileStream fileNamed: 'clone.image'.
	file binary.
	n _ self writeImage: (Array with: Smalltalk).
	time2 _ Time millisecondClockValue.
	Transcript cr; show: n printString , ' bytes written in '
			, (time2 - time1 //1000) printString , ' seconds.'.
	"ctxt pop" "So we can resume"! !

TestInterpreterPlugin class makeSelector: #doPrimitive:withArguments: privilaged: false!
!TestInterpreterPlugin class methodsFor: 'instance creation' stamp: 'ls 7/11/2000 10:30'!
doPrimitive: primitiveName withArguments: argArray
	| proxy plugin |
	proxy _ InterpreterProxy new.
	proxy loadStackFrom: Debugger myContext sender.
	plugin _ self simulatorClass new.
	plugin setInterpreter: proxy.
	^plugin perform: primitiveName asSymbol withArguments: argArray! !

UndefinedObject makeSelector: #say: privilaged: false!
!UndefinedObject methodsFor: 'as yet unclassified' stamp: 'ls 6/19/2000 16:37'!
say: x

	| file |
	file _ FileDirectory default oldFileNamed: 'bob.log'.
	[
		file
			setToEnd;
			nextPutAll: x printString;
			cr;
			cr;
			nextPutAll: Debugger myContext  longStack;
			cr;
			cr.
	] ensure: [file close]! !

Object removeSelector: #tryNamedPrimitive!
Object removeSelector: #tryNamedPrimitive:!
Object removeSelector: #tryNamedPrimitive:with:!
Object removeSelector: #tryNamedPrimitive:with:with:!
Object removeSelector: #tryNamedPrimitive:with:with:with:!
Object removeSelector: #tryNamedPrimitive:with:with:with:with:!
Object removeSelector: #tryNamedPrimitive:with:with:with:with:with:!
Object removeSelector: #tryNamedPrimitive:with:with:with:with:with:with:!
Object removeSelector: #tryNamedPrimitive:with:with:with:with:with:with:with:!
Object removeSelector: #tryPrimitive:withArgs:!
"Postscript: mark all primitives as privilaged"
ProtoObject withAllSubclassesDo: [ :class |
	class methodDict keysAndValuesDo: [ :selector :method |
		(method primitive ~= 0 and: [ method isQuick not ]) ifTrue: [
			class makeSelectorPrivilaged: selector ] ] ]!

