'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 11:58:21 am'!
"Change Set:		betterFlagInCM
Date:			22 June 2000
Author:			Lex Spoon

Adds a flag to CompiledMethod denoting whether the method uses instance variables or not.  This has two uses in the sandbox scheme: class methods using instance variables are unsafe, and methods of proxy objects using instance variables must be analyzed closely"!


!CompiledMethod commentStamp: '<historical>' prior: 0!
My instances are methods suitable for interpretation by the virtual machine.  This is the only class in the system whose instances intermix both indexable pointer fields and indexable integer fields.

	
The current format of a CompiledMethod is as follows:

	header (4 bytes)
	literals (4 bytes each)
	bytecodes  (variable)
	trailer (variable)

The header is a 30-bit integer with the following format:

(index 0)	9 bits:	main part of primitive number   (#primitive)
(index 9)	8 bits:	number of literals (#numLiterals)
(index 17)	1 bit:	whether a large frame size is needed (#frameSize)
(index 18)	6 bits:	number of temporary variables (#numTemps)
(index 24)	4 bits:	number of arguments to the method (#numArgs)
(index 28)	1 bit:	high-bit of primitive number (#primitive)
(index 29)	1 bit:	instance-variables flag (#usesInstanceVariables)



The trailer has two variant formats.  In the first variant, the last byte is at least 252 and the last four bytes represent a source pointer into one of the sources files (see #sourcePointer).  In the second variant, the last byte is less than 252, and the last several bytes are a compressed version of the names of the method's temporary variables.  The number of bytes used for this purpose is the value of the last byte in the method.

!
CompiledMethod makeSelector: #primitive privilaged: false!
!CompiledMethod methodsFor: 'accessing' stamp: 'ls 6/22/2000 14:35'!
primitive
	"Answer the primitive index associated with the receiver.
	Zero indicates that this is not a primitive method.
	We currently allow 10 bits of primitive index, but they are in two places
	for  backward compatibility.  The time to unpack is negligible,
	since the reconstituted full index is stored in the method cache."
	| primBits |
	primBits _ self header bitAnd: 16r100001FF.
	
	^ (primBits bitAnd: 16r1FF) + (primBits bitShift: -19)
! !

CompiledMethod class makeSelector: #newBytes:trailerBytes:nArgs:nTemps:nStack:nLits:primitive:usesInstanceVariables: privilaged: false!
!CompiledMethod class methodsFor: 'instance creation' stamp: 'ls 6/29/2000 14:09'!
newBytes: numberOfBytes trailerBytes: trailer nArgs: nArgs nTemps: nTemps nStack: stackSize nLits: nLits primitive: primitiveIndex usesInstanceVariables: usesInstanceVariables
	"Answer an instance of me. The header is specified by the message 
	arguments. The remaining parts are not as yet determined."
	| largeBit primBits method usesInstanceVariablesBit |
	nTemps > 64 ifTrue:
		[^ self error: 'Cannot compile -- too many temporary variables'].	
	largeBit _ (nTemps + stackSize) > SmallFrame ifTrue: [1] ifFalse: [0].

	"For now the high bit of the primitive no. is in a high bit of the header"
	primBits _ (primitiveIndex bitAnd: 16r1FF) + ((primitiveIndex bitAnd: 16r200) bitShift: 19).

	usesInstanceVariablesBit := usesInstanceVariables ifTrue: [ 1 ] ifFalse: [ 0 ].

	method _ self newMethod: numberOfBytes + trailer size
		header: (nArgs bitShift: 24) +
				(nTemps bitShift: 18) +
				(largeBit bitShift: 17) +
				(nLits bitShift: 9) +
				primBits +
				(usesInstanceVariablesBit bitShift: 29).

	"Copy the source code trailer to the end"
	1 to: trailer size do:
		[:i | method at: method size - trailer size + i put: (trailer at: i)].

	^ method! !

Interpreter makeSelector: #primitiveIndexOf: privilaged: false!
!Interpreter methodsFor: 'compiled methods' stamp: 'ls 6/22/2000 14:35'!
primitiveIndexOf: methodPointer
	"Note: We now have 10 bits of primitive index, but they are in two places
	for temporary backward compatibility.  The time to unpack is negligible,
	since the reconstituted full index is stored in the method cache."
	| primBits |
	primBits _ ((self headerOf: methodPointer) >> 1) bitAnd: 16r100001FF.
	
	^ (primBits bitAnd: 16r1FF) + (primBits >> 19)
! !

SystemDictionary makeSelector: #browseInstVarRefsInProxies privilaged: false!
!SystemDictionary methodsFor: 'browsing' stamp: 'ls 6/22/2000 12:20'!
browseInstVarRefsInProxies
	self browseMessageList: self instVarRefsInProxies name: 'InstVar References in Proxy Classes'! !
SystemDictionary makeSelector: #instVarRefsInProxies privilaged: false!
!SystemDictionary methodsFor: 'retrieving' stamp: 'ls 6/22/2000 12:19'!
instVarRefsInProxies
	"return a list of messages which are implemented in a subclass of RestrictedProxy, and which access an instance variable"
	| messages |
	messages := OrderedCollection new.
	RestrictedProxy  allSubclassesDo: [ :class |
		class selectorsDo: [ :sel |
			(class compiledMethodAt: sel) usesInstanceVariables ifTrue: [
				messages add: class printString, ' ', sel ] ] ].
	^messages! !

