'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 11:56:38 am'!
"Change Set:		classInitialization
Date:			12 August 2000
Author:			Lex Spoon

Fixes up various class initialization methods so that they work inside a sandbox"!

Class makeSelector: #initializeForSandbox privilaged: false!
!Class methodsFor: 'class variables' stamp: 'ls 8/11/2000 18:23'!
initializeForSandbox
	"initialize inside a new sandbox; typically this is the same as the regular #initialize, but for some classes it is different"
	^self initialize! !

BitBlt class makeSelector: #current privilaged: false!
!BitBlt class methodsFor: 'instance creation' stamp: 'ls 8/12/2000 11:44'!
current
	"Return the class currently to be used for BitBlt"
	^Display ifNil: [ self ] ifNotNil: [ Display defaultBitBltClass ]! !

Color class makeSelector: #initialize privilaged: false!
!Color class methodsFor: 'class initialization' stamp: 'ls 8/14/2000 18:41'!
initialize
	"Color initialize"

	"Details: Externally, the red, green, and blue components of color
	are floats in the range [0.0..1.0]. Internally, they are represented
	as integers in the range [0..ComponentMask] packing into a
	small integer to save space and to allow fast hashing and
	equality testing.

	For a general description of color representations for computer
	graphics, including the relationship between the RGB and HSV
	color models used here, see Chapter 17 of Foley and van Dam,
	Fundamentals of Interactive Computer Graphics, Addison-Wesley,
	1982."

	ComponentMask _ 1023.
	HalfComponentMask _ 512.  "used to round up in integer calculations"
	ComponentMax _ 1023.0.  "a Float used to normalize components"
	RedShift _ 20.
	GreenShift _ 10.
	BlueShift _ 0.

	PureRed		 _ self r: 1 g: 0 b: 0.
	PureGreen	 _ self r: 0 g: 1 b: 0.
	PureBlue	 _ self r: 0 g: 0 b: 1.
	PureYellow	 _ self r: 1 g: 1 b: 0.
	PureCyan	 _ self r: 0 g: 1 b: 1.
	PureMagenta _ self r: 1 g: 0 b: 1.

	RandomStream _ Random new.

	self initializeIndexedColors.
	self initializeGrayToIndexMap.
	self initializeHighLights.
	self initializeNames.! !

Date class makeSelector: #initialize privilaged: false!
!Date class methodsFor: 'class initialization' stamp: 'ls 8/10/2000 17:37'!
initialize
	"Initialize class variables representing the names of the months and days and
	the number of seconds, days in each month, and first day of each month."

	MonthNames _
		#(January February March April May June July August September October November December ).
	SecondsInDay _ 24 * 60 * 60.
	DaysInMonth _ #(31 28 31 30 31 30 31 31 30 31 30 31 ).
	FirstDayOfMonth _ #(1 32 60 91 121 152 182 213 244 274 305 335 ).
	WeekDayNames _ #(Monday Tuesday Wednesday Thursday Friday Saturday Sunday ).

	"Date initialize."
! !

Symbol class makeSelector: #initializeForSandbox privilaged: false!
!Symbol class methodsFor: 'class initialization' stamp: 'ls 8/11/2000 18:24'!
initializeForSandbox
	^self! !

Text class makeSelector: #initTextConstants privilaged: false!
!Text class methodsFor: 'class initialization' stamp: 'ls 8/14/2000 18:34'!
initTextConstants 
	"Initialize constants shared by classes associated with text display, e.g., 
	Space, Tab, Cr, Bs, ESC."
		"1/24/96 sw: in exasperation and confusion, changed cmd-g mapping from 231 to 232 to see if I could gain any relief?!!"


	| letter varAndValue tempArray width |
	"CtrlA..CtrlZ, Ctrla..Ctrlz"
	letter _ $A.
 	#(		212 230 228 196 194 226 241 243 214 229 200 217 246 
			245 216 202 210 239 211 240 197 198 209 215 242 231
	 		1 166 228 132 130 12 232 179 150 165 136 153 182 
			14 15 138 17 18 19 11 21 134 145 151 178 167 ) do:
		[:kbd |
		TextConstants at: ('Ctrl', letter asSymbol) asSymbol put: kbd asCharacter.
		letter _ letter == $Z ifTrue: [$a] ifFalse: [(letter asciiValue + 1) asCharacter]].

	varAndValue _ #(
		Space	32
		Tab		9
		CR		13
		Enter	3
		BS		8
		BS2		158
		ESC		160
		Clear 	173
	).

	varAndValue size odd ifTrue: [self notify: 'unpaired text constant'].
	(2 to: varAndValue size by: 2) do:
		[:i | TextConstants at: (varAndValue at: i - 1) put: (varAndValue at: i) asCharacter].

	varAndValue _ #(
		CtrlDigits 			(159 144 143 128 127 129 131 180 149 135)
		CtrlOpenBrackets	(201 7 218 249 219 15)
			"lparen gottn by ctrl-_ = 201; should be 213 but can't type that on Mac"

			"location of non-character stop conditions"
		EndOfRun	257
		CrossedX	258

			"values for alignment"
		LeftFlush	0
		RightFlush	1
		Centered	2
		Justified	3

			"subscripts for a marginTabsArray tuple"
		LeftMarginTab	1
		RightMarginTab	2

			"font faces"
		Basal	0
		Bold	1
		Italic	2

			"in case font doesn't have a width for space character"
			"some plausible numbers-- are they the right ones?"
		DefaultSpace			4
		DefaultTab				24
		DefaultLineGrid			16
		DefaultBaseline			12
		DefaultFontFamilySize	3	"basal, bold, italic"
	).

	varAndValue size odd ifTrue: [self notify: 'unpaired text constant'].
	(2 to: varAndValue size by: 2) do:
		[:i | TextConstants at: (varAndValue at: i - 1) put: (varAndValue at: i)].

	TextConstants at: #DefaultRule	put: Form over.
	TextConstants at: #DefaultMask	put: Color black.

	width _ DisplayCanvas ifNil: [ Display width ]  ifNotNil: [ DisplayCanvas extent x ].
	width _ width max: 720.
	tempArray _ Array new: width // DefaultTab.
	1 to: tempArray size do:
		[:i | tempArray at: i put: DefaultTab * i].
	TextConstants at: #DefaultTabsArray put: tempArray.
	tempArray _ Array new: (width // DefaultTab) // 2.
	1 to: tempArray size do:
		[:i | tempArray at: i put: (Array with: (DefaultTab*i) with: (DefaultTab*i))].
	TextConstants at: #DefaultMarginTabsArray put: tempArray.

"Text initTextConstants "! !
Text class makeSelector: #initializeForSandbox privilaged: false!
!Text class methodsFor: 'class initialization' stamp: 'ls 8/11/2000 18:34'!
initializeForSandbox
	"Text must be initialized specially, because of the way it sets up the pool dictionary TextConstants"
	^self! !

Date initialize!
Color initialize!
Color class removeSelector: #initializeForSandbox!
