'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 11:57:28 am'!
"Change Set:		movePrims2
Date:			27 June 2000
Author:			Lex Spoon

use primitives in their new locations, and remove them from their original locations"!


Array makeSelector: #elementsExchangeIdentityWith: privilaged: false!
!Array methodsFor: 'converting' stamp: 'ls 6/29/2000 14:37'!
elementsExchangeIdentityWith: otherArray
	"This primitive performs a bulk mutation, causing all pointers to the elements of this array to be replaced by pointers to the corresponding elements of otherArray.  At the same time, all pointers to the elements of otherArray are replaced by pointers to the corresponding elements of this array.  The identityHashes remain with the pointers rather than with the objects so that objects in hashed structures should still be properly indexed after the mutation."
	^Smalltalk elementsExchangeIdentity: self with: otherArray


! !
Array makeSelector: #elementsForwardIdentityTo: privilaged: false!
!Array methodsFor: 'converting' stamp: 'ls 6/27/2000 15:26'!
elementsForwardIdentityTo: otherArray
	"This primitive performs a bulk mutation, causing all pointers to the elements of this array to be replaced by pointers to the corresponding elements of otherArray.  The identityHashes remain with the pointers rather than with the objects so that the objects in this array should still be properly indexed in any existing hashed structures after the mutation."
	^Smalltalk elementsForwardIdentity: self with: otherArray
! !

Behavior makeSelector: #allInstancesDo: privilaged: false!
!Behavior methodsFor: 'enumerating' stamp: 'ls 6/27/2000 11:30'!
allInstancesDo: aBlock 
	"Evaluate the argument, aBlock, for each of the current instances of the 
	receiver."
	| inst |
	self ==  UndefinedObject ifTrue: [^ aBlock value: nil].
	inst _ self someInstance.
	[inst == nil]
		whileFalse:
		[aBlock value: inst.
		inst _ self nextInstanceAfter: inst]! !

ClassBuilder makeSelector: #fixObsoleteReferencesTo: privilaged: false!
!ClassBuilder methodsFor: 'private' stamp: 'ls 6/27/2000 11:51'!
fixObsoleteReferencesTo: oldClasses
	"Fix all obsolete references to the given set of outdated classes"
	| obsoleteClasses obj st |
	progress == nil ifFalse:[progress value:'Fixing obsolete class references...'].
	"Prepare a map of obsolete classes"
	obsoleteClasses _ self mapObsoleteClassesToTemps: oldClasses.
	"Sanity check for debugging"
	"oldClasses size = obsoleteClasses size ifFalse:[self error:'Obsolete classes size mismatch']."
	"Fix the methods"
	self fixObsoleteMethodsFrom: oldClasses map: obsoleteClasses.
	"Now search and fix all dangerous objects"
	st _ Smalltalk.
	obj _ st someObject.
	[0 == obj] whileFalse:[
		"Avoid proxies on the disk.  See below."
		obj isInMemory ifTrue:[
			(obj isBehavior and:[obsoleteClasses includesKey: obj superclass]) ifTrue:[
				(obsoleteClasses includesKey: obj) ifFalse:[
						obj superclass: (obsoleteClasses at: obj superclass)]]].
		obj _ st nextObjectAfter: obj.
	].

"Three kinds of ProtoObjects need to be considered:
ObjectTracer and ObjectViewer		OK to test.  They are not Behaviors, and the object they represent will be found and fixed.  Traces may be recorded, but they should be.

ObjectOut and MorphObjectOut		Skip them.  The objects they represent are in SqueakPages, and are not really in this image.  Do not want to bring them all in!!

ImageSegmentRootStub		Skip them.  Might stand for a class, or its segment might contain a class.  But, that class can't be the one being cleaned up now.  If instances of the class being fixed are in a segment, the class is in the outPointers, and will be found normally.
"
! !

ImageSegment makeSelector: #allObjectsDo: privilaged: false!
!ImageSegment methodsFor: 'access' stamp: 'ls 6/27/2000 10:49'!
allObjectsDo: aBlock
	"Enumerate all objects that came from this segment.  NOTE this assumes that the segment was created (and extracted).  After the segment has been installed (install), this method allows you to enumerate its objects."
	| obj |

	endMarker == nil ifTrue: [
		^ self error: 'Just extract and install, don''t writeToFile:'].
	segment size ~= 1 ifTrue: [
		^ self error: 'Vestigial segment size must be 1 (version word)'].

	obj _ Smalltalk nextObjectAfter: segment.  "Start with the next object after the vestigial header"
	[obj == endMarker] whileFalse:  "Stop at the next object after the full segment"
		[aBlock value: obj.
		obj _ Smalltalk nextObjectAfter: obj].  "Step through the objects installed from the segment."! !
ImageSegment makeSelector: #copyFromRoots:sizeHint: privilaged: false!
!ImageSegment methodsFor: 'read/write segment' stamp: 'ls 6/27/2000 10:45'!
copyFromRoots: aRootArray sizeHint: segSizeHint
	"Copy a tree of objects into a WordArray segment.  The copied objects in the segment are not in the normal Squeak space.  If this method yields a very small segment, it is because objects just below the roots are pointed at from the outside.  (See findRogueRootsImSeg: for a *destructive* diagnostic of who is pointing in.)"
	| segmentWordArray outPointerArray segSize rootSet uniqueRoots |
	aRootArray ifNil: [self errorWrongState].
	(Symbol classPool at: #SelectorTables) first first class == WeakArray ifTrue: [
		Symbol useArrayTables].	"Hold onto Symbols.  Done only once."
	rootSet _ IdentitySet new: 150.
	uniqueRoots _ OrderedCollection new.
	1 to: aRootArray size do: [:ii |	"Don't include any roots twice"
		(rootSet includes: (aRootArray at: ii)) 
			ifFalse: [
				uniqueRoots addLast: (aRootArray at: ii).
				rootSet add: (aRootArray at: ii)]
			ifTrue: [userRootCnt ifNotNil: ["adjust the count"
						ii <= userRootCnt ifTrue: [userRootCnt _ userRootCnt - 1]]]].
	arrayOfRoots _ uniqueRoots asArray.
	rootSet _ uniqueRoots _ nil.	"be clean"
	userRootCnt ifNil: [userRootCnt _ arrayOfRoots size].
	arrayOfRoots do: [:aRoot | (aRoot respondsTo: #indexIfCompact) ifTrue: [
		aRoot indexIfCompact > 0 ifTrue: [
			self error: 'Compact class ', aRoot name, ' cannot be a root']].
		aRoot _ nil].	"clean up"
	outPointers _ nil.	"may have used this instance before"
	segSize _ segSizeHint > 0 ifTrue: [segSizeHint *3 //2] ifFalse: [50000].

	["Guess a reasonable segment size"
	segmentWordArray _ WordArrayForSegment new: segSize.
	[outPointerArray _ Array new: segSize // 20] ifError: [
		state _ #tooBig.  ^ self].
	Smalltalk garbageCollect.	"Remove this when everyone is using the new VM"
	(self storeSegmentFor: arrayOfRoots
					into: segmentWordArray
					outPointers: outPointerArray) == nil]
		whileTrue:
			["Double the segment size and try again"
			segmentWordArray _ outPointerArray _ nil.
			segSize _ segSize * 2].
	segment _ segmentWordArray.
	outPointers _ outPointerArray.
	state _ #activeCopy.
	endMarker _ Smalltalk nextObjectAfter: segment. 	"for enumeration of objects"
	endMarker == 0 ifTrue: [endMarker _ 'End' clone].
! !
ImageSegment makeSelector: #install privilaged: false!
!ImageSegment methodsFor: 'read/write segment' stamp: 'ls 6/27/2000 10:45'!
install
	"This operation retrieves the segment if necessary from file storage, installs it in memory, and replaces (using become:) all the root stubs with the reconstructed roots of the segment."

	| newRoots |
	state = #onFile ifTrue: [self readFromFile].
	state = #onFileWithSymbols ifTrue: [self readFromFileWithSymbols.
		endMarker _ Smalltalk nextObjectAfter: segment. 	"for enumeration of objects"
		endMarker == 0 ifTrue: [endMarker _ 'End' clone]].
	(state = #active) | (state = #imported) ifFalse: [self errorWrongState].
	newRoots _ self loadSegmentFrom: segment outPointers: outPointers.
	state = #imported 
		ifTrue: ["just came in from exported file"
			arrayOfRoots _ newRoots]
		ifFalse: [
			arrayOfRoots elementsForwardIdentityTo: newRoots].
	state _ #inactive.
	self beep.	"Don't use Squeak sound here."
! !
ImageSegment makeSelector: #readFromFile privilaged: false!
!ImageSegment methodsFor: 'read/write segment' stamp: 'ls 6/27/2000 10:45'!
readFromFile
	"Read in a simple segment.  Use folder of this image, even if remembered as previous location of this image"

	| ff realName |
	realName _ self class folder, FileDirectory slash, self localName.
	ff _ FileStream oldFileNamed: realName.
	segment _ ff nextWordsInto: (WordArrayForSegment new: ff size//4).
	endMarker _ Smalltalk nextObjectAfter: segment. 	"for enumeration of objects"
	endMarker == 0 ifTrue: [endMarker _ 'End' clone].
	ff close.
	state _ #active! !
ImageSegment makeSelector: #comeFullyUpOnReload: privilaged: false!
!ImageSegment methodsFor: 'fileIn/Out' stamp: 'ls 6/27/2000 10:45'!
comeFullyUpOnReload: smartRefStream
	"fix up the objects in the segment that changed size.  An object in the segment is the wrong size for the modern version of the class.  Construct a fake class that is the old size.  Replace the modern class with the old one in outPointers.  Load the segment.  Traverse the instances, making new instances by copying fields, and running conversion messages.  Keep the new instances.  Bulk forward become the old to the new.  Let go of the fake objects and classes.
	After the install (below), arrayOfRoots is filled in.  Globalize new classes.  Caller may want to do some special install on certain objects in arrayOfRoots.
	May want to write the segment out to disk in its new form."

	| mapFakeClassesToReal fakes goods bads perfect ccFixups real insts receiverClasses |
	mapFakeClassesToReal _ smartRefStream reshapedClassesIn: outPointers.
		"Dictionary of just the ones that change shape.  Substitute them in outPointers."
	ccFixups _ self remapCompactClasses: mapFakeClassesToReal 
				refStrm: smartRefStream.
	ccFixups ifFalse: [^ self error: 'A class in the file is not compatible'].
	endMarker _ Smalltalk nextObjectAfter: segment. 	"for enumeration of objects"
	endMarker == 0 ifTrue: [endMarker _ 'End' clone].
	arrayOfRoots _ self loadSegmentFrom: segment outPointers: outPointers.
		"Can't use install.  Not ready for rehashSets"
	mapFakeClassesToReal isEmpty ifFalse: [
		fakes _ mapFakeClassesToReal keys.
		goods _ OrderedCollection new.
		bads _ OrderedCollection new.
		fakes do: [:aFakeClass | 
			real _ mapFakeClassesToReal at: aFakeClass.
			(real indexIfCompact > 0) "and there is a fake class"
				ifFalse: ["normal case"
					aFakeClass allInstancesDo: [:misShapen | 
						perfect _ smartRefStream convert: misShapen to: real.
						(bads includes: misShapen) ifFalse: [
							bads add: misShapen.
							goods add: perfect]]]
				ifTrue: ["instances have the wrong class.  Fix them before anyone notices."
					insts _ OrderedCollection new.
					self allObjectsDo: [:obj | obj class == real ifTrue: [insts add: obj]].
					insts do: [:misShapen | 
						perfect _ smartRefStream convert: misShapen to: real.
						(bads includes: misShapen) ifFalse: [
							bads add: misShapen.
							goods add: perfect]]]].

		bads size > 0 ifTrue: [
			bads asArray elementsForwardIdentityTo: goods asArray]].
	receiverClasses _ self restoreEndianness.		"rehash sets"
	smartRefStream checkFatalReshape: receiverClasses.

	"Classes in this segment."
	arrayOfRoots do: [:aRoot | 
		(aRoot isKindOf: Project) ifTrue: [
			Project allInstancesDo: [:pp | pp ~~ aRoot ifTrue: [
				pp name = aRoot name ifTrue: [
					aRoot projectChangeSet name: ChangeSet defaultName]]]].
		aRoot class class == Metaclass ifTrue: [
			self declare: aRoot]].

	mapFakeClassesToReal isEmpty ifFalse: [
		fakes do: [:aFake | 
			aFake indexIfCompact > 0 ifTrue: [aFake becomeUncompact].
			aFake removeFromSystemUnlogged].
		SystemOrganization removeEmptyCategories].
	"^ self"! !
ImageSegment makeSelector: #rehashSets privilaged: false!
!ImageSegment methodsFor: 'fileIn/Out' stamp: 'ls 6/27/2000 10:45'!
rehashSets
	"I have just been brought in and converted to live objects.  Find all Sets and Dictionaries in the newly created objects and rehash them.  Segment is near then end of memory, since is was newly brought in (and a new object created for it).
	Also, collect all classes of receivers of blocks.  Return them.  Caller will check if they have been reshaped."

	| object sets receiverClasses inSeg |
	object _ segment.
	sets _ OrderedCollection new.
		"have to collect them, because Dictionary makes a copy, and that winds up at the end of memory and gets rehashed and makes another one."
	receiverClasses _ IdentitySet new.
	inSeg _ true.
	[object _ Smalltalk nextObjectAfter: object.  
		object == endMarker ifTrue: [inSeg _ false].	"off end"
		object isInMemory ifTrue: [
			(object isKindOf: Set) ifTrue: [sets add: object].
			object class == BlockContext ifTrue: [inSeg ifTrue: [
					receiverClasses add: object receiver class]].	
			object class == MethodContext ifTrue: [inSeg ifTrue: [
					receiverClasses add: object receiver class]].	
			]. 
		object == 0] whileFalse.
	sets do: [:each | each rehash].	"our purpose"
	^ receiverClasses	"our secondary job"
! !
ImageSegment makeSelector: #restoreEndianness privilaged: false!
!ImageSegment methodsFor: 'fileIn/Out' stamp: 'ls 6/27/2000 10:45'!
restoreEndianness
	"Fix endianness (byte order) of any objects not already fixed.
	I have just been brought in and converted to live objects.  Find all Sets and Dictionaries in the newly created objects and rehash them.  Segment is near then end of memory, since is was newly brought in (and a new object created for it).
	Also, collect all classes of receivers of blocks.  Return them.  Caller will check if they have been reshaped."

	| object sets receiverClasses inSeg reorder |
	object _ segment.
	sets _ OrderedCollection new.
		"have to collect them, because Dictionary makes a copy, and that winds up at the end of memory and gets rehashed and makes another one."
	receiverClasses _ IdentitySet new.
	reorder _ (Smalltalk endianness) ~~ 
		((segment first bitShift: -24) asCharacter == $d ifTrue: [#big] ifFalse: [#little]).
	inSeg _ true.
	[object _ Smalltalk nextObjectAfter: object.  "all the way to the end of memory to catch remade objects"
		object == endMarker ifTrue: [inSeg _ false].	"off end"
		object isInMemory ifTrue: [
			(object isKindOf: Set) ifTrue: [sets add: object].
			object class == BlockContext ifTrue: [inSeg ifTrue: [
					receiverClasses add: object receiver class]].	
			object class == MethodContext ifTrue: [inSeg ifTrue: [
					receiverClasses add: object receiver class]].	
			(reorder and: [(object class == SoundBuffer) & inSeg]) ifTrue: [object swapHalves]]. 
		object == 0] whileFalse.
	sets do: [:each | each rehash].	"our purpose"
	^ receiverClasses	"our secondary job"
! !

Project makeSelector: #storeToMakeRoom privilaged: false!
!Project methodsFor: 'file in/out' stamp: 'ls 6/27/2000 11:30'!
storeToMakeRoom
	"Write out enough projects to fulfill the space goals.
	Include the size of the project about to come in."

	| params memoryEnd goalFree cnt gain proj skip tried |
	GoalFreePercent ifNil: [GoalFreePercent _ 33].
	GoalNotMoreThan ifNil: [GoalNotMoreThan _ 20000000].
	params _ Smalltalk getVMParameters.
	memoryEnd	_ params at: 3.
"	youngSpaceEnd	_ params at: 2.
	free _ memoryEnd - youngSpaceEnd.
"
	goalFree _ GoalFreePercent asFloat / 100.0 * memoryEnd.
	goalFree _ goalFree min: GoalNotMoreThan.
	world isInMemory ifFalse: ["enough room to bring it in"
		goalFree _ goalFree + (self projectParameters at: #segmentSize ifAbsent: [0])].
	cnt _ 30.
	gain _ Smalltalk garbageCollectMost.
	"skip a random number of projects that are in memory"
	proj _ self.  skip _ 6 atRandom.
	[proj _ (Project nextInstanceAfter: proj) ifNil: [Project someInstance].
		proj world isInMemory ifTrue: [skip _ skip - 1].
		skip > 0] whileTrue.
	cnt _ 0.  tried _ 0.

	[gain > goalFree] whileFalse: [
		proj _ (Project nextInstanceAfter: proj) ifNil: [Project someInstance].
		proj storeSegment ifTrue: ["Yes, did send its morphs to the disk"
			gain _ gain + (proj projectParameters at: #segmentSize 
						ifAbsent: [20000]).	"a guess"
			self beep.
			(cnt _ cnt + 1) > 5 ifTrue: [^ self]].	"put out 5 at most"
		(tried _ tried + 1) > 23 ifTrue: [^ self]].	"don't get stuck in a loop"! !

SystemDictionary makeSelector: #allObjectsDo: privilaged: false!
!SystemDictionary methodsFor: 'retrieving' stamp: 'ls 6/27/2000 10:46'!
allObjectsDo: aBlock 
	"Evaluate the argument, aBlock, for each object in the system
	 excluding SmallIntegers."

	| object |
	object _ self someObject.
	[0 == object] whileFalse: [
		aBlock value: object.
		object _ self nextObjectAfter: object.
	].! !
SystemDictionary makeSelector: #pointersTo:except: privilaged: false!
!SystemDictionary methodsFor: 'retrieving' stamp: 'ls 6/27/2000 10:46'!
pointersTo: anObject except: objectsToExclude
	"Find all occurrences in the system of pointers to the argument anObject. Remove objects in the exclusion list from the results."

	| results anObj |
	Smalltalk garbageCollect.
	"big collection shouldn't grow, so it's contents array is always the same"
	results _ OrderedCollection new: 1000.

	"allObjectsDo: is expanded inline to keep spurious
	 method and block contexts out of the results"
	anObj _ self someObject.
	[0 == anObj] whileFalse: [
		anObj isInMemory ifTrue: [
			(anObj pointsTo: anObject) ifTrue: [
				"exclude the results collector and contexts in call chain"
				((anObj ~~ results collector) and:
				 [(anObj ~~ objectsToExclude) and:
				 [(anObj ~~ Debugger myContext) and:
				 [(anObj ~~ Debugger myContext sender) and:
				 [anObj ~~ Debugger myContext sender sender]]]])
					 ifTrue: [ results add: anObj ].
			]].
		anObj _ self nextObjectAfter: anObj.
	].
	objectsToExclude do: [ :obj | results removeAllSuchThat: [ :el | el == obj]].

	^ results asArray
! !

SmallInteger removeSelector: #nextInstance!
PseudoContext removeSelector: #nextObject!
Object removeSelector: #someObject!
ProtoObject removeSelector: #nextInstance!
ProtoObject removeSelector: #nextObject!
